@@ -9,7 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2008-2018 University of Houston. All rights reserved.
+ * Copyright (c) 2008-2019 University of Houston. All rights reserved.
  * Copyright (c) 2017-2018 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2017      IBM Corporation. All rights reserved.
@@ -91,6 +91,12 @@ int mca_common_ompio_set_view (ompio_file_t *fh,
         fh->f_datarep = NULL;
     }
 
+    if (NULL != fh->f_file_convertor) {
+        opal_convertor_cleanup (fh->f_file_convertor);
+        free (fh->f_file_convertor);
+        fh->f_file_convertor = NULL;
+    }
+    
     /* Reset the flags first */
     if ( fh->f_flags & OMPIO_CONTIGUOUS_FVIEW ) {
         fh->f_flags &= ~OMPIO_CONTIGUOUS_FVIEW;
@@ -98,9 +104,24 @@ int mca_common_ompio_set_view (ompio_file_t *fh,
     if ( fh->f_flags & OMPIO_UNIFORM_FVIEW ) {
         fh->f_flags &= ~OMPIO_UNIFORM_FVIEW;
     }
+    if ( fh->f_flags & OMPIO_DATAREP_NATIVE ) {
+        fh->f_flags &= ~OMPIO_DATAREP_NATIVE;
+    }
     fh->f_datarep = strdup (datarep);
-    datatype_duplicate (filetype, &fh->f_orig_filetype );
 
+    if ( !(strcmp(datarep, "external32") && strcmp(datarep, "EXTERNAL32"))) {
+        fh->f_file_convertor = malloc ( sizeof(struct opal_convertor_t) );
+        if ( NULL == fh->f_file_convertor ) {
+            return OMPI_ERR_OUT_OF_RESOURCE;
+        }
+        opal_convertor_clone (ompi_mpi_external32_convertor, fh->f_file_convertor, 0);
+    }
+    else {
+        fh->f_file_convertor = opal_convertor_create (opal_local_arch, 0);
+        fh->f_flags |= OMPIO_DATAREP_NATIVE;
+    }
+    
+    datatype_duplicate (filetype, &fh->f_orig_filetype );
     opal_datatype_get_extent(&filetype->super, &lb, &ftype_extent);
     opal_datatype_type_size (&filetype->super, &ftype_size);
 
@@ -129,6 +150,7 @@ int mca_common_ompio_set_view (ompio_file_t *fh,
                                       1,
                                       NULL,
                                       &max_data,
+                                      fh->f_file_convertor,
                                       &fh->f_decoded_iov,
                                       &fh->f_iov_count);
 
@@ -328,36 +350,28 @@ exit:
 
 OMPI_MPI_OFFSET_TYPE get_contiguous_chunk_size (ompio_file_t *fh, int flag)
 {
-    int uniform = 0;
     OMPI_MPI_OFFSET_TYPE avg[3] = {0,0,0};
     OMPI_MPI_OFFSET_TYPE global_avg[3] = {0,0,0};
     int i = 0;
 
-    /* This function does two things: first, it determines the average data chunk
-    ** size in the file view for each process and across all processes.
-    ** Second, it establishes whether the view across all processes is uniform.
-    ** By definition, uniform means:
-    ** 1. the file view of each process has the same number of contiguous sections
-    ** 2. each section in the file view has exactly the same size
+    /* This function determines the average data chunk
+    ** size in the file view for each process and across all processes,
+    ** and the avg. file_view size across processes.
     */
 
     if ( flag  ) {
         global_avg[0] = MCA_IO_DEFAULT_FILE_VIEW_SIZE;
+        fh->f_avg_view_size = fh->f_view_size;
     }
     else {
         for (i=0 ; i<(int)fh->f_iov_count ; i++) {
             avg[0] += fh->f_decoded_iov[i].iov_len;
-            if (i && 0 == uniform) {
-                if (fh->f_decoded_iov[i].iov_len != fh->f_decoded_iov[i-1].iov_len) {
-                    uniform = 1;
-                }
-            }
         }
         if ( 0 != fh->f_iov_count ) {
             avg[0] = avg[0]/fh->f_iov_count;
         }
         avg[1] = (OMPI_MPI_OFFSET_TYPE) fh->f_iov_count;
-        avg[2] = (OMPI_MPI_OFFSET_TYPE) uniform;
+        avg[2] = (OMPI_MPI_OFFSET_TYPE) fh->f_view_size;
         
         fh->f_comm->c_coll->coll_allreduce (avg,
                                             global_avg,
@@ -368,37 +382,7 @@ OMPI_MPI_OFFSET_TYPE get_contiguous_chunk_size (ompio_file_t *fh, int flag)
                                             fh->f_comm->c_coll->coll_allreduce_module);
         global_avg[0] = global_avg[0]/fh->f_size;
         global_avg[1] = global_avg[1]/fh->f_size;
-        
-#if 0 
-        /* Disabling the feature since we are not using it anyway. Saves us one allreduce operation. */
-        int global_uniform=0;
-        
-        if ( global_avg[0] == avg[0] &&
-             global_avg[1] == avg[1] &&
-             0 == avg[2]             &&
-             0 == global_avg[2] ) {
-            uniform = 0;
-        }
-        else {
-            uniform = 1;
-        }
-        
-        /* second confirmation round to see whether all processes agree
-        ** on having a uniform file view or not
-        */
-        fh->f_comm->c_coll->coll_allreduce (&uniform,
-                                            &global_uniform,
-                                            1,
-                                            MPI_INT,
-                                            MPI_MAX,
-                                            fh->f_comm,
-                                            fh->f_comm->c_coll->coll_allreduce_module);
-        
-        if ( 0 == global_uniform  ){
-            /* yes, everybody agrees on having a uniform file view */
-            fh->f_flags |= OMPIO_UNIFORM_FVIEW;
-        }
-#endif
+        fh->f_avg_view_size =  global_avg[2]/fh->f_size;
     }
 
     return global_avg[0];
