@@ -31,13 +31,10 @@
 
 #include "common_ompio.h"
 #include "common_ompio_request.h"
+#include "common_ompio_buffer.h"
 #include <unistd.h>
 #include <math.h>
 
-#if OPAL_CUDA_SUPPORT
-#include "common_ompio_cuda.h"
-#endif
-
 int mca_common_ompio_file_write (ompio_file_t *fh,
 			       const void *buf,
 			       int count,
@@ -72,16 +69,34 @@ int mca_common_ompio_file_write (ompio_file_t *fh,
         return ret;
     }
 
+    bool need_to_copy = false;
+
 #if OPAL_CUDA_SUPPORT
     int is_gpu, is_managed;
     mca_common_ompio_check_gpu_buf ( fh, buf, &is_gpu, &is_managed);
     if ( is_gpu && !is_managed ) {
+        need_to_copy = true;
+    }
+#endif
+
+    if ( !( fh->f_flags & OMPIO_DATAREP_NATIVE ) &&
+         !(datatype == &ompi_mpi_byte.dt  ||
+           datatype == &ompi_mpi_char.dt   )) {
+        /* only need to copy if any of these conditions are given:
+           1. buffer is an unmanaged CUDA buffer (checked above).
+           2. Datarepresentation is anything other than 'native' and
+           3. datatype is not byte or char (i.e it does require some actual
+              work to be done e.g. for external32.
+        */
+        need_to_copy = true;
+    }         
+    
+    if ( need_to_copy ) {
         size_t pos=0;
         char *tbuf=NULL;
         opal_convertor_t convertor;
         
-        OMPIO_CUDA_PREPARE_BUF(fh,buf,count,datatype,tbuf,&convertor,max_data,decoded_iov,iov_count);        
-        
+        OMPIO_PREPARE_BUF(fh,buf,count,datatype,tbuf,&convertor,max_data,decoded_iov,iov_count);     
         opal_convertor_pack (&convertor, decoded_iov, &iov_count, &pos );
         opal_convertor_cleanup ( &convertor);
     }
@@ -91,22 +106,19 @@ int mca_common_ompio_file_write (ompio_file_t *fh,
                                           count,
                                           buf,
                                           &max_data,
+                                          fh->f_mem_convertor,
                                           &decoded_iov,
                                           &iov_count);
     }
-#else
-    mca_common_ompio_decode_datatype (fh,
-                                      datatype,
-                                      count,
-                                      buf,
-                                      &max_data,
-                                      &decoded_iov,
-                                      &iov_count);
-#endif
+
     if ( 0 < max_data && 0 == fh->f_iov_count  ) {
         if ( MPI_STATUS_IGNORE != status ) {
             status->_ucount = 0;
         }
+        if (NULL != decoded_iov) {
+            free (decoded_iov);
+            decoded_iov = NULL;
+        }
         return OMPI_SUCCESS;
     }
 
@@ -134,7 +146,9 @@ int mca_common_ompio_file_write (ompio_file_t *fh,
                                           &i,
                                           &j,
                                           &total_bytes_written, 
-                                          &spc);
+                                          &spc,
+                                          &fh->f_io_array,
+                                          &fh->f_num_of_io_entries);
 
         if (fh->f_num_of_io_entries) {
             ret_code =fh->f_fbtl->fbtl_pwritev (fh);
@@ -149,11 +163,11 @@ int mca_common_ompio_file_write (ompio_file_t *fh,
             fh->f_io_array = NULL;
         }
     }
-#if OPAL_CUDA_SUPPORT
-    if ( is_gpu && !is_managed ) {
+
+    if ( need_to_copy ) {
         mca_common_ompio_release_buf (fh, decoded_iov->iov_base);
     }
-#endif
+
 
     if (NULL != decoded_iov) {
         free (decoded_iov);
@@ -228,16 +242,34 @@ int mca_common_ompio_file_iwrite (ompio_file_t *fh,
         int i = 0; /* index into the decoded iovec of the buffer */
         int j = 0; /* index into the file vie iovec */
 
+        bool need_to_copy = false;
+
 #if OPAL_CUDA_SUPPORT
         int is_gpu, is_managed;
         mca_common_ompio_check_gpu_buf ( fh, buf, &is_gpu, &is_managed);
         if ( is_gpu && !is_managed ) {
+            need_to_copy = true;
+        }
+#endif
+
+        if ( !( fh->f_flags & OMPIO_DATAREP_NATIVE ) &&
+             !(datatype == &ompi_mpi_byte.dt  ||
+               datatype == &ompi_mpi_char.dt   )) {
+            /* only need to copy if any of these conditions are given:
+               1. buffer is an unmanaged CUDA buffer (checked above).
+               2. Datarepresentation is anything other than 'native' and
+               3. datatype is not byte or char (i.e it does require some actual
+               work to be done e.g. for external32.
+            */
+            need_to_copy = true;
+        }         
+        
+        if ( need_to_copy ) {
             size_t pos=0;
             char *tbuf=NULL;
             opal_convertor_t convertor;
-
-            OMPIO_CUDA_PREPARE_BUF(fh,buf,count,datatype,tbuf,&convertor,max_data,decoded_iov,iov_count);        
             
+            OMPIO_PREPARE_BUF(fh,buf,count,datatype,tbuf,&convertor,max_data,decoded_iov,iov_count);                    
             opal_convertor_pack (&convertor, decoded_iov, &iov_count, &pos );
             opal_convertor_cleanup (&convertor);
 
@@ -250,23 +282,21 @@ int mca_common_ompio_file_iwrite (ompio_file_t *fh,
                                               count,
                                               buf,
                                               &max_data,
+                                              fh->f_mem_convertor,
                                               &decoded_iov,
                                               &iov_count);
         }
-#else
-        mca_common_ompio_decode_datatype (fh,
-                                          datatype,
-                                          count,
-                                          buf,
-                                          &max_data,
-                                          &decoded_iov,
-                                          &iov_count);
-#endif
+
         if ( 0 < max_data && 0 == fh->f_iov_count  ) {
             ompio_req->req_ompi.req_status.MPI_ERROR = OMPI_SUCCESS;
             ompio_req->req_ompi.req_status._ucount = 0;
             ompi_request_complete (&ompio_req->req_ompi, false);
             *request = (ompi_request_t *) ompio_req;
+            if (NULL != decoded_iov) {
+                free (decoded_iov);
+                decoded_iov = NULL;
+            }
+
             return OMPI_SUCCESS;
         }
 
@@ -283,7 +313,9 @@ int mca_common_ompio_file_iwrite (ompio_file_t *fh,
                                           &i,
                                           &j,
                                           &total_bytes_written, 
-                                          &spc);
+                                          &spc,
+                                          &fh->f_io_array,
+                                          &fh->f_num_of_io_entries);
         
         if (fh->f_num_of_io_entries) {
             fh->f_fbtl->fbtl_ipwritev (fh, (ompi_request_t *) ompio_req);
@@ -348,6 +380,62 @@ int mca_common_ompio_file_iwrite_at (ompio_file_t *fh,
 
 /* Collective operations                                          */
 /******************************************************************/
+int mca_common_ompio_file_write_all (ompio_file_t *fh,
+                                     const void *buf,
+                                     int count,
+                                     struct ompi_datatype_t *datatype,
+                                     ompi_status_public_t *status)
+{
+    int ret = OMPI_SUCCESS;
+    
+    if ( !( fh->f_flags & OMPIO_DATAREP_NATIVE ) &&
+         !(datatype == &ompi_mpi_byte.dt  ||
+           datatype == &ompi_mpi_char.dt   )) {
+        /* No need to check for GPU buffer for collective I/O.
+           Most algorithms first copy data to aggregators, and send/recv
+           to/from GPU buffers works if ompi was compiled was GPU support.
+           
+           If the individual fcoll component is used: there are no aggregators 
+           in that concept. However, since they call common_ompio_file_write, 
+           CUDA buffers are handled by that routine.
+
+           Thus, we only check for
+           1. Datarepresentation is anything other than 'native' and
+           2. datatype is not byte or char (i.e it does require some actual
+              work to be done e.g. for external32.
+        */
+        size_t pos=0, max_data=0;
+        char *tbuf=NULL;
+        opal_convertor_t convertor;
+        struct iovec *decoded_iov = NULL;
+        uint32_t iov_count = 0;
+        
+        OMPIO_PREPARE_BUF(fh,buf,count,datatype,tbuf,&convertor,max_data,decoded_iov,iov_count);     
+        opal_convertor_pack (&convertor, decoded_iov, &iov_count, &pos );
+        opal_convertor_cleanup ( &convertor);
+
+        ret = fh->f_fcoll->fcoll_file_write_all (fh,
+                                                 decoded_iov->iov_base,
+                                                 decoded_iov->iov_len,
+                                                 MPI_BYTE,
+                                                 status);
+
+
+        mca_common_ompio_release_buf (fh, decoded_iov->iov_base);
+        if (NULL != decoded_iov) {
+            free (decoded_iov);
+            decoded_iov = NULL;
+        }
+    }
+    else {
+        ret = fh->f_fcoll->fcoll_file_write_all (fh,
+                                                 buf,
+                                                 count,
+                                                 datatype,
+                                                 status);
+    }
+    return ret;
+}
 
 int mca_common_ompio_file_write_at_all (ompio_file_t *fh,
 				      OMPI_MPI_OFFSET_TYPE offset,
@@ -361,30 +449,23 @@ int mca_common_ompio_file_write_at_all (ompio_file_t *fh,
     mca_common_ompio_file_get_position (fh, &prev_offset );
 
     mca_common_ompio_set_explicit_offset (fh, offset);
-    ret = fh->f_fcoll->fcoll_file_write_all (fh,
-                                             buf,
-                                             count,
-                                             datatype,
-                                             status);
-
+    ret = mca_common_ompio_file_write_all (fh,
+                                           buf,
+                                           count,
+                                           datatype,
+                                           status);
+    
     mca_common_ompio_set_explicit_offset (fh, prev_offset);
     return ret;
 }
 
-int mca_common_ompio_file_iwrite_at_all (ompio_file_t *fp,
-				       OMPI_MPI_OFFSET_TYPE offset,
-				       const void *buf,
-				       int count,
-				       struct ompi_datatype_t *datatype,
-				       ompi_request_t **request)
+int mca_common_ompio_file_iwrite_all (ompio_file_t *fp,
+                                      const void *buf,
+                                      int count,
+                                      struct ompi_datatype_t *datatype,
+                                      ompi_request_t **request)
 {
-
     int ret = OMPI_SUCCESS;
-    OMPI_MPI_OFFSET_TYPE prev_offset;
-
-    mca_common_ompio_file_get_position (fp, &prev_offset );
-
-    mca_common_ompio_set_explicit_offset (fp, offset);
 
     if ( NULL != fp->f_fcoll->fcoll_file_iwrite_all ) {
 	ret = fp->f_fcoll->fcoll_file_iwrite_all (fp,
@@ -400,18 +481,40 @@ int mca_common_ompio_file_iwrite_at_all (ompio_file_t *fp,
 	ret = mca_common_ompio_file_iwrite ( fp, buf, count, datatype, request );
     }
 
+    return ret;
+}
+
+
+int mca_common_ompio_file_iwrite_at_all (ompio_file_t *fp,
+				       OMPI_MPI_OFFSET_TYPE offset,
+				       const void *buf,
+				       int count,
+				       struct ompi_datatype_t *datatype,
+				       ompi_request_t **request)
+{
+
+    int ret = OMPI_SUCCESS;
+    OMPI_MPI_OFFSET_TYPE prev_offset;
+
+    mca_common_ompio_file_get_position (fp, &prev_offset );
+    mca_common_ompio_set_explicit_offset (fp, offset);
+    
+    ret = mca_common_ompio_file_iwrite_all ( fp, buf, count, datatype, request );
+
     mca_common_ompio_set_explicit_offset (fp, prev_offset);
     return ret;
 }
 
 
+
 /* Helper function used by both read and write operations     */
 /**************************************************************/
 
 int mca_common_ompio_build_io_array ( ompio_file_t *fh, int index, int cycles,
                                       size_t bytes_per_cycle, size_t  max_data, uint32_t iov_count,
                                       struct iovec *decoded_iov, int *ii, int *jj, size_t *tbw, 
-                                      size_t *spc)
+                                      size_t *spc, mca_common_ompio_io_array_t **io_array,
+                                      int *num_io_entries)
 {
     ptrdiff_t disp;
     int block = 1;
@@ -424,7 +527,9 @@ int mca_common_ompio_build_io_array ( ompio_file_t *fh, int index, int cycles,
     int k = 0; /* index into the io_array */
     int i = *ii;
     int j = *jj;
-
+    mca_common_ompio_io_array_t *f_io_array=NULL;
+    int f_num_io_entries=0;
+    
     sum_previous_length = fh->f_position_in_file_view;
 
     if ((index == cycles-1) && (max_data % bytes_per_cycle)) {
@@ -434,9 +539,9 @@ int mca_common_ompio_build_io_array ( ompio_file_t *fh, int index, int cycles,
 	bytes_to_write_in_cycle = bytes_per_cycle;
     }
 
-    fh->f_io_array = (mca_common_ompio_io_array_t *)malloc
+    f_io_array = (mca_common_ompio_io_array_t *)malloc
 	(OMPIO_IOVEC_INITIAL_SIZE * sizeof (mca_common_ompio_io_array_t));
-    if (NULL == fh->f_io_array) {
+    if (NULL == f_io_array) {
 	opal_output(1, "OUT OF MEMORY\n");
 	return OMPI_ERR_OUT_OF_RESOURCE;
     }
@@ -445,10 +550,10 @@ int mca_common_ompio_build_io_array ( ompio_file_t *fh, int index, int cycles,
 	/* reallocate if needed  */
 	if (OMPIO_IOVEC_INITIAL_SIZE*block <= k) {
 	    block ++;
-	    fh->f_io_array = (mca_common_ompio_io_array_t *)realloc
-		(fh->f_io_array, OMPIO_IOVEC_INITIAL_SIZE *
+	    f_io_array = (mca_common_ompio_io_array_t *)realloc
+		(f_io_array, OMPIO_IOVEC_INITIAL_SIZE *
 		 block * sizeof (mca_common_ompio_io_array_t));
-	    if (NULL == fh->f_io_array) {
+	    if (NULL == f_io_array) {
 		opal_output(1, "OUT OF MEMORY\n");
 		return OMPI_ERR_OUT_OF_RESOURCE;
 	    }
@@ -462,15 +567,15 @@ int mca_common_ompio_build_io_array ( ompio_file_t *fh, int index, int cycles,
 
 	disp = (ptrdiff_t)decoded_iov[i].iov_base +
 	    (total_bytes_written - sum_previous_counts);
-	fh->f_io_array[k].memory_address = (IOVBASE_TYPE *)disp;
+	f_io_array[k].memory_address = (IOVBASE_TYPE *)disp;
 
 	if (decoded_iov[i].iov_len -
 	    (total_bytes_written - sum_previous_counts) >=
 	    bytes_to_write_in_cycle) {
-	    fh->f_io_array[k].length = bytes_to_write_in_cycle;
+	    f_io_array[k].length = bytes_to_write_in_cycle;
 	}
 	else {
-	    fh->f_io_array[k].length =  decoded_iov[i].iov_len -
+	    f_io_array[k].length =  decoded_iov[i].iov_len -
 		(total_bytes_written - sum_previous_counts);
 	}
 
@@ -492,36 +597,36 @@ int mca_common_ompio_build_io_array ( ompio_file_t *fh, int index, int cycles,
 
 	disp = (ptrdiff_t)fh->f_decoded_iov[j].iov_base +
 	    (fh->f_total_bytes - sum_previous_length);
-	fh->f_io_array[k].offset = (IOVBASE_TYPE *)(intptr_t)(disp + fh->f_offset);
+	f_io_array[k].offset = (IOVBASE_TYPE *)(intptr_t)(disp + fh->f_offset);
 
 	if (! (fh->f_flags & OMPIO_CONTIGUOUS_FVIEW)) {
 	    if (fh->f_decoded_iov[j].iov_len -
 		(fh->f_total_bytes - sum_previous_length)
-		< fh->f_io_array[k].length) {
-		fh->f_io_array[k].length = fh->f_decoded_iov[j].iov_len -
+		< f_io_array[k].length) {
+		f_io_array[k].length = fh->f_decoded_iov[j].iov_len -
 		    (fh->f_total_bytes - sum_previous_length);
 	    }
 	}
 
-	total_bytes_written += fh->f_io_array[k].length;
-	fh->f_total_bytes += fh->f_io_array[k].length;
-	bytes_to_write_in_cycle -= fh->f_io_array[k].length;
+	total_bytes_written += f_io_array[k].length;
+	fh->f_total_bytes += f_io_array[k].length;
+	bytes_to_write_in_cycle -= f_io_array[k].length;
 	k = k + 1;
     }
     fh->f_position_in_file_view = sum_previous_length;
     fh->f_index_in_file_view = j;
-    fh->f_num_of_io_entries = k;
+    f_num_io_entries = k;
 
 #if 0
     if (fh->f_rank == 0) {
 	int d;
-	printf("*************************** %d\n", fh->f_num_of_io_entries);
+	printf("*************************** %d\n", f_num_io_entries);
 
-	for (d=0 ; d<fh->f_num_of_io_entries ; d++) {
+	for (d=0 ; d<f_num_of_entries ; d++) {
 	    printf(" ADDRESS: %p  OFFSET: %p   LENGTH: %d prev_count=%ld prev_length=%ld\n",
-		   fh->f_io_array[d].memory_address,
-		   fh->f_io_array[d].offset,
-		   fh->f_io_array[d].length, 
+		   f_io_array[d].memory_address,
+		   f_io_array[d].offset,
+		   f_io_array[d].length, 
                    sum_previous_counts, sum_previous_length);
 	}
     }
@@ -530,7 +635,9 @@ int mca_common_ompio_build_io_array ( ompio_file_t *fh, int index, int cycles,
     *jj = j;
     *tbw = total_bytes_written;
     *spc = sum_previous_counts;
-
+    *io_array = f_io_array;
+    *num_io_entries = f_num_io_entries;
+    
     return OMPI_SUCCESS;
 }
 
