@@ -10,6 +10,9 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008-2015 University of Houston. All rights reserved.
+ * Copyright (c) 2018      Cisco Systems, Inc.  All rights reserved
+ * Copyright (c) 2018      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -76,7 +79,7 @@ int mca_fbtl_posix_component_init_query(bool enable_progress_threads,
 }
 
 struct mca_fbtl_base_module_1_0_0_t *
-mca_fbtl_posix_component_file_query (mca_io_ompio_file_t *fh, int *priority) {
+mca_fbtl_posix_component_file_query (ompio_file_t *fh, int *priority) {
    *priority = mca_fbtl_posix_priority;
 
    if (UFS == fh->f_fstype) {
@@ -88,7 +91,7 @@ mca_fbtl_posix_component_file_query (mca_io_ompio_file_t *fh, int *priority) {
    return &posix;
 }
 
-int mca_fbtl_posix_component_file_unquery (mca_io_ompio_file_t *file) {
+int mca_fbtl_posix_component_file_unquery (ompio_file_t *file) {
    /* This function might be needed for some purposes later. for now it
     * does not have anything to do since there are no steps which need
     * to be undone if this module is not selected */
@@ -96,7 +99,7 @@ int mca_fbtl_posix_component_file_unquery (mca_io_ompio_file_t *file) {
    return OMPI_SUCCESS;
 }
 
-int mca_fbtl_posix_module_init (mca_io_ompio_file_t *file) {
+int mca_fbtl_posix_module_init (ompio_file_t *file) {
 
 #if defined (FBTL_POSIX_HAVE_AIO)
     long val = sysconf(_SC_AIO_MAX);
@@ -108,7 +111,7 @@ int mca_fbtl_posix_module_init (mca_io_ompio_file_t *file) {
 }
 
 
-int mca_fbtl_posix_module_finalize (mca_io_ompio_file_t *file) {
+int mca_fbtl_posix_module_finalize (ompio_file_t *file) {
     return OMPI_SUCCESS;
 }
 
@@ -116,7 +119,7 @@ bool mca_fbtl_posix_progress ( mca_ompio_request_t *req)
 {
     bool ret=false;
 #if defined (FBTL_POSIX_HAVE_AIO)
-    int i=0, lcount=0, ret_code;
+    int i=0, lcount=0, ret_code=0;
     mca_fbtl_posix_request_data_t *data=(mca_fbtl_posix_request_data_t *)req->req_data;
     off_t start_offset, end_offset, total_length;
 
