@@ -23,12 +23,12 @@ mca_mtl_ofi_module_t ompi_mtl_ofi = {
         ompi_mtl_ofi_del_procs,
         ompi_mtl_ofi_finalize,
 
-        ompi_mtl_ofi_send,
-        ompi_mtl_ofi_isend,
-        ompi_mtl_ofi_irecv,
-        ompi_mtl_ofi_iprobe,
-        ompi_mtl_ofi_imrecv,
-        ompi_mtl_ofi_improbe,
+         NULL,
+         NULL,
+         NULL,
+         NULL,
+         ompi_mtl_ofi_imrecv,
+         NULL,
 
         ompi_mtl_ofi_cancel,
         ompi_mtl_ofi_add_comm,
@@ -98,9 +98,10 @@ ompi_mtl_ofi_add_procs(struct mca_mtl_base_module_t *mtl,
                               (void**)&ep_name,
                               &size);
         if (OMPI_SUCCESS != ret) {
-            opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
-                                "%s:%d: modex_recv failed: %d\n",
-                                __FILE__, __LINE__, ret);
+            opal_show_help("help-mtl-ofi.txt", "modex failed",
+                           true, ompi_process_info.nodename,
+			   procs[i]->super.proc_hostname,
+			   opal_strerror(ret), ret);
             goto bail;
         }
         memcpy(&ep_names[i*namelen], ep_name, namelen);
@@ -111,7 +112,7 @@ ompi_mtl_ofi_add_procs(struct mca_mtl_base_module_t *mtl,
      */
     count = fi_av_insert(ompi_mtl_ofi.av, ep_names, nprocs, fi_addrs, 0, NULL);
     if ((count < 0) || (nprocs != (size_t)count)) {
-        opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
+        opal_output_verbose(1, opal_common_ofi.output,
                             "%s:%d: fi_av_insert failed: %d\n",
                             __FILE__, __LINE__, count);
         ret = OMPI_ERROR;
@@ -124,7 +125,7 @@ ompi_mtl_ofi_add_procs(struct mca_mtl_base_module_t *mtl,
     for (i = 0; i < nprocs; ++i) {
         endpoint = OBJ_NEW(mca_mtl_ofi_endpoint_t);
         if (NULL == endpoint) {
-            opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
+            opal_output_verbose(1, opal_common_ofi.output,
                                 "%s:%d: mtl/ofi: could not allocate endpoint"
                                 " structure\n",
                                 __FILE__, __LINE__);
@@ -169,7 +170,7 @@ ompi_mtl_ofi_del_procs(struct mca_mtl_base_module_t *mtl,
             endpoint = procs[i]->proc_endpoints[OMPI_PROC_ENDPOINT_TAG_MTL];
             ret = fi_av_remove(ompi_mtl_ofi.av, &endpoint->peer_fiaddr, 1, 0);
             if (ret) {
-                opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
+                opal_output_verbose(1, opal_common_ofi.output,
                         "%s:%d: fi_av_remove failed: %s\n", __FILE__, __LINE__, fi_strerror(errno));
                 return ret;
             }
