@@ -3,8 +3,9 @@
  * Copyright (c) 2011-2017 Sandia National Laboratories.  All rights reserved.
  * Copyright (c) 2015      Los Alamos National Security, LLC.  All rights
  *                         reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2015-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2016-2017 IBM Corporation. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -143,7 +144,7 @@ int ompi_osc_portals4_put(const void *origin_addr,
                           int origin_count,
                           struct ompi_datatype_t *origin_dt,
                           int target,
-                          OPAL_PTRDIFF_TYPE target_disp,
+                          ptrdiff_t target_disp,
                           int target_count,
                           struct ompi_datatype_t *target_dt,
                           struct ompi_win_t *win);
@@ -152,7 +153,7 @@ int ompi_osc_portals4_get(void *origin_addr,
                           int origin_count,
                           struct ompi_datatype_t *origin_dt,
                           int target,
-                          OPAL_PTRDIFF_TYPE target_disp,
+                          ptrdiff_t target_disp,
                           int target_count,
                           struct ompi_datatype_t *target_dt,
                           struct ompi_win_t *win);
@@ -161,7 +162,7 @@ int ompi_osc_portals4_accumulate(const void *origin_addr,
                                  int origin_count,
                                  struct ompi_datatype_t *origin_dt,
                                  int target,
-                                 OPAL_PTRDIFF_TYPE target_disp,
+                                 ptrdiff_t target_disp,
                                  int target_count,
                                  struct ompi_datatype_t *target_dt,
                                  struct ompi_op_t *op,
@@ -172,14 +173,14 @@ int ompi_osc_portals4_compare_and_swap(const void *origin_addr,
                                        void *result_addr,
                                        struct ompi_datatype_t *dt,
                                        int target,
-                                       OPAL_PTRDIFF_TYPE target_disp,
+                                       ptrdiff_t target_disp,
                                        struct ompi_win_t *win);
 
 int ompi_osc_portals4_fetch_and_op(const void *origin_addr,
                                    void *result_addr,
                                    struct ompi_datatype_t *dt,
                                    int target,
-                                   OPAL_PTRDIFF_TYPE target_disp,
+                                   ptrdiff_t target_disp,
                                    struct ompi_op_t *op,
                                    struct ompi_win_t *win);
 
@@ -190,7 +191,7 @@ int ompi_osc_portals4_get_accumulate(const void *origin_addr,
                                      int result_count,
                                      struct ompi_datatype_t *result_datatype,
                                      int target_rank,
-                                     OPAL_PTRDIFF_TYPE target_disp,
+                                     ptrdiff_t target_disp,
                                      int target_count,
                                      struct ompi_datatype_t *target_datatype,
                                      struct ompi_op_t *op,
@@ -200,7 +201,7 @@ int ompi_osc_portals4_rput(const void *origin_addr,
                            int origin_count,
                            struct ompi_datatype_t *origin_dt,
                            int target,
-                           OPAL_PTRDIFF_TYPE target_disp,
+                           ptrdiff_t target_disp,
                            int target_count,
                            struct ompi_datatype_t *target_dt,
                            struct ompi_win_t *win,
@@ -210,7 +211,7 @@ int ompi_osc_portals4_rget(void *origin_addr,
                            int origin_count,
                            struct ompi_datatype_t *origin_dt,
                            int target,
-                           OPAL_PTRDIFF_TYPE target_disp,
+                           ptrdiff_t target_disp,
                            int target_count,
                            struct ompi_datatype_t *target_dt,
                            struct ompi_win_t *win,
@@ -220,7 +221,7 @@ int ompi_osc_portals4_raccumulate(const void *origin_addr,
                                   int origin_count,
                                   struct ompi_datatype_t *origin_dt,
                                   int target,
-                                  OPAL_PTRDIFF_TYPE target_disp,
+                                  ptrdiff_t target_disp,
                                   int target_count,
                                   struct ompi_datatype_t *target_dt,
                                   struct ompi_op_t *op,
@@ -234,7 +235,7 @@ int ompi_osc_portals4_rget_accumulate(const void *origin_addr,
                                       int result_count,
                                       struct ompi_datatype_t *result_datatype,
                                       int target_rank,
-                                      OPAL_PTRDIFF_TYPE target_disp,
+                                      ptrdiff_t target_disp,
                                       int target_count,
                                       struct ompi_datatype_t *target_datatype,
                                       struct ompi_op_t *op,
@@ -281,8 +282,8 @@ int ompi_osc_portals4_flush_local(int target,
                                   struct ompi_win_t *win);
 int ompi_osc_portals4_flush_local_all(struct ompi_win_t *win);
 
-int ompi_osc_portals4_set_info(struct ompi_win_t *win, struct ompi_info_t *info);
-int ompi_osc_portals4_get_info(struct ompi_win_t *win, struct ompi_info_t **info_used);
+int ompi_osc_portals4_set_info(struct ompi_win_t *win, struct opal_info_t *info);
+int ompi_osc_portals4_get_info(struct ompi_win_t *win, struct opal_info_t **info_used);
 
 static inline int
 ompi_osc_portals4_complete_all(ompi_osc_portals4_module_t *module)
