@@ -10,7 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2013-2016 University of Houston. All rights reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2015-2018 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2016-2017 IBM Corporation. All rights reserved.
  * $COPYRIGHT$
@@ -34,11 +34,11 @@ BEGIN_C_DECLS
 int mca_sharedfp_lockedfile_component_init_query(bool enable_progress_threads,
                                                  bool enable_mpi_threads);
 struct mca_sharedfp_base_module_1_0_0_t *
-        mca_sharedfp_lockedfile_component_file_query (mca_io_ompio_file_t *file, int *priority);
-int mca_sharedfp_lockedfile_component_file_unquery (mca_io_ompio_file_t *file);
+        mca_sharedfp_lockedfile_component_file_query (ompio_file_t *file, int *priority);
+int mca_sharedfp_lockedfile_component_file_unquery (ompio_file_t *file);
 
-int mca_sharedfp_lockedfile_module_init (mca_io_ompio_file_t *file);
-int mca_sharedfp_lockedfile_module_finalize (mca_io_ompio_file_t *file);
+int mca_sharedfp_lockedfile_module_init (ompio_file_t *file);
+int mca_sharedfp_lockedfile_module_finalize (ompio_file_t *file);
 
 extern int mca_sharedfp_lockedfile_priority;
 extern int mca_sharedfp_lockedfile_verbose;
@@ -51,52 +51,52 @@ OMPI_MODULE_DECLSPEC extern mca_sharedfp_base_component_2_0_0_t mca_sharedfp_loc
  */
 /*IMPORANT: Update here when implementing functions from sharedfp API*/
 
-int mca_sharedfp_lockedfile_seek (mca_io_ompio_file_t *fh,
+int mca_sharedfp_lockedfile_seek (ompio_file_t *fh,
                                   OMPI_MPI_OFFSET_TYPE offset, int whence);
-int mca_sharedfp_lockedfile_get_position (mca_io_ompio_file_t *fh,
+int mca_sharedfp_lockedfile_get_position (ompio_file_t *fh,
                                           OMPI_MPI_OFFSET_TYPE * offset);
 int mca_sharedfp_lockedfile_file_open (struct ompi_communicator_t *comm,
                                        const char* filename,
                                        int amode,
                                        struct opal_info_t *info,
-                                       mca_io_ompio_file_t *fh);
-int mca_sharedfp_lockedfile_file_close (mca_io_ompio_file_t *fh);
-int mca_sharedfp_lockedfile_read (mca_io_ompio_file_t *fh,
+                                       ompio_file_t *fh);
+int mca_sharedfp_lockedfile_file_close (ompio_file_t *fh);
+int mca_sharedfp_lockedfile_read (ompio_file_t *fh,
                                   void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
-int mca_sharedfp_lockedfile_read_ordered (mca_io_ompio_file_t *fh,
+int mca_sharedfp_lockedfile_read_ordered (ompio_file_t *fh,
                                           void *buf, int count, struct ompi_datatype_t *datatype,
                                           ompi_status_public_t *status
                                           );
-int mca_sharedfp_lockedfile_read_ordered_begin (mca_io_ompio_file_t *fh,
+int mca_sharedfp_lockedfile_read_ordered_begin (ompio_file_t *fh,
                                                  void *buf,
                                                  int count,
                                                  struct ompi_datatype_t *datatype);
-int mca_sharedfp_lockedfile_read_ordered_end (mca_io_ompio_file_t *fh,
+int mca_sharedfp_lockedfile_read_ordered_end (ompio_file_t *fh,
                                                void *buf,
                                                ompi_status_public_t *status);
-int mca_sharedfp_lockedfile_iread (mca_io_ompio_file_t *fh,
+int mca_sharedfp_lockedfile_iread (ompio_file_t *fh,
                                     void *buf,
                                     int count,
                                     struct ompi_datatype_t *datatype,
                                     ompi_request_t **request);
-int mca_sharedfp_lockedfile_write (mca_io_ompio_file_t *fh,
+int mca_sharedfp_lockedfile_write (ompio_file_t *fh,
                                    const void *buf,
                                    int count,
                                    struct ompi_datatype_t *datatype,
                                    ompi_status_public_t *status);
-int mca_sharedfp_lockedfile_write_ordered (mca_io_ompio_file_t *fh,
+int mca_sharedfp_lockedfile_write_ordered (ompio_file_t *fh,
                                            const void *buf,
                                            int count,
                                            struct ompi_datatype_t *datatype,
                                            ompi_status_public_t *status);
-int mca_sharedfp_lockedfile_write_ordered_begin (mca_io_ompio_file_t *fh,
+int mca_sharedfp_lockedfile_write_ordered_begin (ompio_file_t *fh,
                                                  const void *buf,
                                                  int count,
                                                  struct ompi_datatype_t *datatype);
-int mca_sharedfp_lockedfile_write_ordered_end (mca_io_ompio_file_t *fh,
+int mca_sharedfp_lockedfile_write_ordered_end (ompio_file_t *fh,
                                                const void *buf,
                                                ompi_status_public_t *status);
-int mca_sharedfp_lockedfile_iwrite (mca_io_ompio_file_t *fh,
+int mca_sharedfp_lockedfile_iwrite (ompio_file_t *fh,
                                     const void *buf,
                                     int count,
                                     struct ompi_datatype_t *datatype,
