@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2010 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2016 The University of Tennessee and The University
+ * Copyright (c) 2004-2018 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart,
@@ -13,9 +13,11 @@
  * Copyright (c) 2006-2008 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2010-2012 Oracle and/or its affiliates.  All rights reserved.
  * Copyright (c) 2012      Oak Ridge National Labs.  All rights reserved.
- * Copyright (c) 2016      Los Alamos National Security, LLC. All rights
+ * Copyright (c) 2016-2017 Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2016      Mellanox Technologies. All rights reserved.
+ * Copyright (c) 2016      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -29,8 +31,7 @@
 #include "ompi/request/request_default.h"
 #include "ompi/request/grequest.h"
 
-#include "ompi/mca/pml/base/pml_base_request.h"
-
+#include "ompi/mca/crcp/crcp.h"
 
 int ompi_request_default_wait(
     ompi_request_t ** req_ptr,
@@ -40,6 +41,7 @@ int ompi_request_default_wait(
 
     ompi_request_wait_completion(req);
 
+    OMPI_CRCP_REQUEST_COMPLETE(req);
 
     /* return status.  If it's a generalized request, we *have* to
        invoke the query_fn, even if the user procided STATUS_IGNORE.
@@ -89,10 +91,17 @@ int ompi_request_default_wait_any(size_t count,
     ompi_request_t *request=NULL;
     ompi_wait_sync_t sync;
 
+    if (OPAL_UNLIKELY(0 == count)) {
+        *index = MPI_UNDEFINED;
+        return OMPI_SUCCESS;
+    }
+
     WAIT_SYNC_INIT(&sync, 1);
 
     num_requests_null_inactive = 0;
     for (i = 0; i < count; i++) {
+        void *_tmp_ptr = REQUEST_PENDING;
+
         request = requests[i];
 
         /* Check for null or completed persistent request. For
@@ -103,7 +112,7 @@ int ompi_request_default_wait_any(size_t count,
             continue;
         }
 
-        if( !OPAL_ATOMIC_CMPSET_PTR(&request->req_complete, REQUEST_PENDING, &sync) ) {
+        if( !OPAL_ATOMIC_COMPARE_EXCHANGE_STRONG_PTR(&request->req_complete, &_tmp_ptr, &sync) ) {
             assert(REQUEST_COMPLETE(request));
             completed = i;
             *index = i;
@@ -129,6 +138,8 @@ int ompi_request_default_wait_any(size_t count,
      * user.
      */
     for(i = completed-1; (i+1) > 0; i--) {
+        void *tmp_ptr = &sync;
+
         request = requests[i];
 
         if( request->req_state == OMPI_REQUEST_INACTIVE ) {
@@ -139,7 +150,7 @@ int ompi_request_default_wait_any(size_t count,
          * Otherwise, the request has been completed meanwhile, and it
          * has been atomically marked as REQUEST_COMPLETE.
          */
-        if( !OPAL_ATOMIC_CMPSET_PTR(&request->req_complete, &sync, REQUEST_PENDING) ) {
+        if( !OPAL_ATOMIC_COMPARE_EXCHANGE_STRONG_PTR(&request->req_complete, &tmp_ptr, REQUEST_PENDING) ) {
             *index = i;
         }
     }
@@ -154,6 +165,11 @@ int ompi_request_default_wait_any(size_t count,
 
     request = requests[*index];
     assert( REQUEST_COMPLETE(request) );
+#if OPAL_ENABLE_FT_CR == 1
+    if( opal_cr_is_enabled ) {
+        OMPI_CRCP_REQUEST_COMPLETE(request);
+    }
+#endif
     /* Per note above, we have to call gen request query_fn even
        if STATUS_IGNORE was provided */
     if (OMPI_REQUEST_GEN == request->req_type) {
@@ -192,9 +208,15 @@ int ompi_request_default_wait_all( size_t count,
     int mpi_error = OMPI_SUCCESS;
     ompi_wait_sync_t sync;
 
+    if (OPAL_UNLIKELY(0 == count)) {
+        return OMPI_SUCCESS;
+    }
+
     WAIT_SYNC_INIT(&sync, count);
     rptr = requests;
     for (i = 0; i < count; i++) {
+        void *_tmp_ptr = REQUEST_PENDING;
+
         request = *rptr++;
 
         if( request->req_state == OMPI_REQUEST_INACTIVE ) {
@@ -202,7 +224,7 @@ int ompi_request_default_wait_all( size_t count,
             continue;
         }
 
-        if (!OPAL_ATOMIC_CMPSET_PTR(&request->req_complete, REQUEST_PENDING, &sync)) {
+        if (!OPAL_ATOMIC_COMPARE_EXCHANGE_STRONG_PTR(&request->req_complete, &_tmp_ptr, &sync)) {
             if( OPAL_UNLIKELY( MPI_SUCCESS != request->req_status.MPI_ERROR ) ) {
                 failed++;
             }
@@ -230,6 +252,8 @@ int ompi_request_default_wait_all( size_t count,
     if (MPI_STATUSES_IGNORE != statuses) {
         /* fill out status and free request if required */
         for( i = 0; i < count; i++, rptr++ ) {
+            void *_tmp_ptr = &sync;
+
             request = *rptr;
 
             if( request->req_state == OMPI_REQUEST_INACTIVE ) {
@@ -244,7 +268,7 @@ int ompi_request_default_wait_all( size_t count,
                  * mark the request as pending then it is neither failed nor complete, and
                  * we must stop altering it.
                  */
-                if( OPAL_ATOMIC_CMPSET_PTR(&request->req_complete, &sync, REQUEST_PENDING ) ) {
+                if( OPAL_ATOMIC_COMPARE_EXCHANGE_STRONG_PTR(&request->req_complete, &_tmp_ptr, REQUEST_PENDING ) ) {
                     /*
                      * Per MPI 2.2 p 60:
                      * Allows requests to be marked as MPI_ERR_PENDING if they are
@@ -258,6 +282,10 @@ int ompi_request_default_wait_all( size_t count,
             }
             assert( REQUEST_COMPLETE(request) );
 
+            if( opal_cr_is_enabled) {
+                OMPI_CRCP_REQUEST_COMPLETE(request);
+            }
+
             if (OMPI_REQUEST_GEN == request->req_type) {
                 ompi_grequest_invoke_query(request, &request->req_status);
             }
@@ -286,6 +314,8 @@ int ompi_request_default_wait_all( size_t count,
         int rc;
         /* free request if required */
         for( i = 0; i < count; i++, rptr++ ) {
+            void *_tmp_ptr = &sync;
+
             request = *rptr;
 
             if( request->req_state == OMPI_REQUEST_INACTIVE ) {
@@ -300,7 +330,7 @@ int ompi_request_default_wait_all( size_t count,
                 /* If the request is still pending due to a failed request
                  * then skip it in this loop.
                  */
-                 if( OPAL_ATOMIC_CMPSET_PTR(&request->req_complete, &sync, REQUEST_PENDING ) ) {
+                 if( OPAL_ATOMIC_COMPARE_EXCHANGE_STRONG_PTR(&request->req_complete, &_tmp_ptr, REQUEST_PENDING ) ) {
                     /*
                      * Per MPI 2.2 p 60:
                      * Allows requests to be marked as MPI_ERR_PENDING if they are
@@ -313,6 +343,10 @@ int ompi_request_default_wait_all( size_t count,
             }
             assert( REQUEST_COMPLETE(request) );
 
+            if( opal_cr_is_enabled) {
+                OMPI_CRCP_REQUEST_COMPLETE(request);
+            }
+
             /* Per note above, we have to call gen request query_fn
                even if STATUSES_IGNORE was provided */
             if (OMPI_REQUEST_GEN == request->req_type) {
@@ -354,13 +388,18 @@ int ompi_request_default_wait_some(size_t count,
                                    int * indices,
                                    ompi_status_public_t * statuses)
 {
-    size_t num_requests_null_inactive=0, num_requests_done=0;
+    size_t num_requests_null_inactive, num_requests_done, num_active_reqs;
     int rc = MPI_SUCCESS;
     ompi_request_t **rptr = NULL;
     ompi_request_t *request = NULL;
     ompi_wait_sync_t sync;
     size_t sync_sets = 0, sync_unsets = 0;
-    
+
+    if (OPAL_UNLIKELY(0 == count)) {
+        *outcount = MPI_UNDEFINED;
+        return OMPI_SUCCESS;
+    }
+
     WAIT_SYNC_INIT(&sync, 1);
 
     *outcount = 0;
@@ -368,7 +407,10 @@ int ompi_request_default_wait_some(size_t count,
     rptr = requests;
     num_requests_null_inactive = 0;
     num_requests_done = 0;
+    num_active_reqs = 0;
     for (size_t i = 0; i < count; i++, rptr++) {
+        void *_tmp_ptr = REQUEST_PENDING;
+
         request = *rptr;
         /*
          * Check for null or completed persistent request.
@@ -378,14 +420,14 @@ int ompi_request_default_wait_some(size_t count,
             num_requests_null_inactive++;
             continue;
         }
-
-        if( !OPAL_ATOMIC_CMPSET_PTR(&request->req_complete, REQUEST_PENDING, &sync) ) {
+        indices[num_active_reqs] = OPAL_ATOMIC_COMPARE_EXCHANGE_STRONG_PTR(&request->req_complete, &_tmp_ptr, &sync);
+        if( !indices[num_active_reqs] ) {
             /* If the request is completed go ahead and mark it as such */
             assert( REQUEST_COMPLETE(request) );
             num_requests_done++;
         }
+        num_active_reqs++;
     }
-    sync_sets = count - num_requests_null_inactive - num_requests_done;
 
     if(num_requests_null_inactive == count) {
         *outcount = MPI_UNDEFINED;
@@ -394,6 +436,7 @@ int ompi_request_default_wait_some(size_t count,
         return rc;
     }
 
+    sync_sets = num_active_reqs - num_requests_done;
     if( 0 == num_requests_done ) {
         /* One completed request is enough to satisfy the some condition */
         SYNC_WAIT(&sync);
@@ -404,24 +447,36 @@ int ompi_request_default_wait_some(size_t count,
 
     rptr = requests;
     num_requests_done = 0;
+    num_active_reqs = 0;
     for (size_t i = 0; i < count; i++, rptr++) {
+        void *_tmp_ptr = &sync;
+
         request = *rptr;
 
         if( request->req_state == OMPI_REQUEST_INACTIVE ) {
             continue;
         }
-        /* Atomically mark the request as pending. If this succeed
-         * then the request was not completed, and it is now marked as
-         * pending. Otherwise, the request is complete )either it was
-         * before or it has been meanwhile). The major drawback here
-         * is that we will do all the atomics operations in all cases.
+        /* Here we have 3 possibilities:
+         * a) request was found completed in the first loop
+         *    => ( indices[i] == 0 )
+         * b) request was completed between first loop and this check
+         *    => ( indices[i] == 1 ) and we can NOT atomically mark the 
+         *    request as pending.
+         * c) request wasn't finished yet
+         *    => ( indices[i] == 1 ) and we CAN  atomically mark the 
+         *    request as pending.
+         * NOTE that in any case (i >= num_requests_done) as latter grows
+         * either slowly (in case of partial completion)
+         * OR in parallel with `i` (in case of full set completion)  
          */
-        if( !OPAL_ATOMIC_CMPSET_PTR(&request->req_complete, &sync, REQUEST_PENDING) ) {
-            indices[num_requests_done] = i;
-            num_requests_done++;
+        if( !indices[num_active_reqs] ) {
+            indices[num_requests_done++] = i;
+        } else if( !OPAL_ATOMIC_COMPARE_EXCHANGE_STRONG_PTR(&request->req_complete, &_tmp_ptr, REQUEST_PENDING) ) {
+            indices[num_requests_done++] = i;
         }
+        num_active_reqs++;
     }
-    sync_unsets = count - num_requests_null_inactive - num_requests_done;
+    sync_unsets = num_active_reqs - num_requests_done;
 
     if( sync_sets == sync_unsets ){
         /* nobody knows about us,
@@ -437,6 +492,13 @@ int ompi_request_default_wait_some(size_t count,
     for (size_t i = 0; i < num_requests_done; i++) {
         request = requests[indices[i]];
         assert( REQUEST_COMPLETE(request) );
+
+#if OPAL_ENABLE_FT_CR == 1
+        if( opal_cr_is_enabled) {
+            OMPI_CRCP_REQUEST_COMPLETE(request);
+        }
+#endif
+
         /* Per note above, we have to call gen request query_fn even
            if STATUS_IGNORE was provided */
         if (OMPI_REQUEST_GEN == request->req_type) {
