@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2016 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -434,9 +434,11 @@ static inline int ompi_request_complete(ompi_request_t* request, bool with_signa
 {
     int rc = 0;
 
-    if( NULL != request->req_complete_cb) {
-        rc = request->req_complete_cb( request );
+    if(NULL != request->req_complete_cb) {
+        /* Set the request cb to NULL to allow resetting in the callback */
+        ompi_request_complete_fn_t fct = request->req_complete_cb;
         request->req_complete_cb = NULL;
+        rc = fct( request );
     }
 
     if (0 == rc) {
@@ -457,6 +459,21 @@ static inline int ompi_request_complete(ompi_request_t* request, bool with_signa
     return OMPI_SUCCESS;
 }
 
+static inline int ompi_request_set_callback(ompi_request_t* request,
+                                            ompi_request_complete_fn_t cb,
+                                            void* cb_data)
+{
+    request->req_complete_cb_data = cb_data;
+    request->req_complete_cb = cb;
+    /* If request is completed and the callback is not called, need to call callback */
+    if ((NULL != request->req_complete_cb) && (request->req_complete == REQUEST_COMPLETED)) {
+        ompi_request_complete_fn_t fct = request->req_complete_cb;
+        request->req_complete_cb = NULL;
+        return fct( request );
+    }
+    return OMPI_SUCCESS;
+}
+
 END_C_DECLS
 
 #endif
