@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2006 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2017 The University of Tennessee and The University
+ * Copyright (c) 2004-2019 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2006 High Performance Computing Center Stuttgart,
@@ -281,15 +281,23 @@ int32_t opal_datatype_add( opal_datatype_t* pdtBase, const opal_datatype_t* pdtA
     if( (pdtAdd->flags & (OPAL_DATATYPE_FLAG_PREDEFINED | OPAL_DATATYPE_FLAG_DATA)) == (OPAL_DATATYPE_FLAG_PREDEFINED | OPAL_DATATYPE_FLAG_DATA) ) {
         if( NULL != pdtBase->ptypes )
             pdtBase->ptypes[pdtAdd->id] += count;
-        pLast->elem.common.type      = pdtAdd->id;
-        pLast->elem.count            = count;
-        pLast->elem.disp             = disp;
-        pLast->elem.extent           = extent;
-        pdtBase->desc.used++;
+
         pLast->elem.common.flags     = pdtAdd->flags & ~(OPAL_DATATYPE_FLAG_COMMITTED);
-        if( (extent != (ptrdiff_t)pdtAdd->size) && (count > 1) ) {  /* gaps around the datatype */
-            pLast->elem.common.flags &= ~(OPAL_DATATYPE_FLAG_CONTIGUOUS | OPAL_DATATYPE_FLAG_NO_GAPS);
+        pLast->elem.common.type      = pdtAdd->id;
+        pLast->elem.disp             = disp;
+        pLast->elem.extent           = (ptrdiff_t)count * extent;
+        /* assume predefined datatypes without extent, aka. contiguous */
+        pLast->elem.count            = 1;
+        pLast->elem.blocklen         = count;
+        if( extent != (ptrdiff_t)pdtAdd->size ) {  /* not contiguous: let's fix */
+            pLast->elem.count            = count;
+            pLast->elem.blocklen         = 1;
+            pLast->elem.extent           = extent;
+            if( count > 1 ) {  /* gaps around the predefined datatype */
+                pLast->elem.common.flags &= ~(OPAL_DATATYPE_FLAG_CONTIGUOUS | OPAL_DATATYPE_FLAG_NO_GAPS);
+            }
         }
+        pdtBase->desc.used++;
     } else {
         /* keep trace of the total number of basic datatypes in the datatype definition */
         pdtBase->loops += pdtAdd->loops;
@@ -299,13 +307,49 @@ int32_t opal_datatype_add( opal_datatype_t* pdtBase, const opal_datatype_t* pdtA
             for( i = OPAL_DATATYPE_FIRST_TYPE; i < OPAL_DATATYPE_MAX_PREDEFINED; i++ )
                 if( pdtAdd->ptypes[i] != 0 ) pdtBase->ptypes[i] += (count * pdtAdd->ptypes[i]);
         }
-        if( (1 == pdtAdd->desc.used) && (extent == (pdtAdd->ub - pdtAdd->lb)) &&
-            (extent == pdtAdd->desc.desc[0].elem.extent) ){
+        if( 1 == pdtAdd->desc.used ) {
             pLast->elem        = pdtAdd->desc.desc[0].elem;
-            pLast->elem.count *= count;
             pLast->elem.disp  += disp;
+            if( 1 == count ) {
+                /* Extent only has a meaning when there are multiple elements. Bail out */
+            } else if( 1 == pLast->elem.count ) {
+                /* The size and true_extent of the added datatype are identical, signaling a datatype
+                 * that is mostly contiguous with the exception of the initial and final gaps. These
+                 * gaps do not matter here as they will amended (the initial gaps being shifted by the
+                 * new displacement and the final gap being replaced with the new gap
+                 */
+                if( pdtAdd->desc.desc[0].elem.extent == extent ) {
+                    /* pure bliss everything is fully contiguous and we can collapse
+                     * everything by updating the blocklen and extent
+                     */
+                    pLast->elem.blocklen *= count;
+                    pLast->elem.extent   *= count;
+                } else {
+                    pLast->elem.count = count;
+                    pLast->elem.extent = extent;
+                }
+            } else if( extent == ((ptrdiff_t)pLast->elem.count * pLast->elem.extent) ) {
+                /* It's just a repetition of the same element, increase the count */
+                /* We need to protect against the case where the multiplication below results in a
+                 * number larger than the max uint32_t. In the unlikely situation where that's the case
+                 * we should not try to optimize the item further but instead fall back and build a loop
+                 * around it.
+                 */
+                uint32_t cnt = pLast->elem.count * count;
+                if( cnt < pLast->elem.count ) {
+                    goto build_loop;
+                }
+                pLast->elem.count = cnt;  /* we're good, merge the elements */
+            } else {
+                /* No luck here, no optimization can be applied. Fall back to the
+                 * normal case where we add a loop around the datatype.
+                 */
+                goto build_loop;
+            }
             pdtBase->desc.used++;
         } else {
+
+build_loop:
             /* if the extent of the datatype is the same as the extent of the loop
              * description of the datatype then we simply have to update the main loop.
              */
