@@ -4,8 +4,8 @@
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
- * Copyright (c) 2015-2017 Research Organization for Information Science
- *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2015-2018 Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -43,50 +43,45 @@ static inline void _predefined_data( const dt_elem_desc_t* ELEM,
                                      const opal_datatype_t* DATATYPE,
                                      unsigned char* SOURCE_BASE,
                                      size_t TOTAL_COUNT,
-                                     uint32_t COUNT,
+                                     size_t COUNT,
                                      unsigned char* SOURCE,
                                      unsigned char* DESTINATION,
                                      size_t* SPACE )
 {
-    uint32_t _copy_count = (COUNT);
-    size_t _copy_blength;
     const ddt_elem_desc_t* _elem = &((ELEM)->elem);
     unsigned char* _source = (SOURCE) + _elem->disp;
     unsigned char* _destination = (DESTINATION) + _elem->disp;
+    size_t do_now = _elem->count, do_now_bytes;
 
-    _copy_blength = opal_datatype_basicDatatypes[_elem->common.type]->size;
+    assert( (COUNT) == (do_now * _elem->blocklen));
 
-    if( _copy_blength == (uint32_t)_elem->extent ) {
-        _copy_blength *= _copy_count;
-        OPAL_DATATYPE_SAFEGUARD_POINTER( _source, _copy_blength, (SOURCE_BASE),
-                                    (DATATYPE), (TOTAL_COUNT) );
-        /* the extent and the size of the basic datatype are equals */
-        DO_DEBUG( opal_output( 0, "copy 1. %s( %p, %p, %lu ) => space %lu\n",
-                               STRINGIFY(MEM_OP_NAME), (void*)_destination, (void*)_source, (unsigned long)_copy_blength, (unsigned long)(*(SPACE)) ); );
-        MEM_OP( _destination, _source, _copy_blength );
-        _source      += _copy_blength;
-        _destination += _copy_blength;
-    } else {
-        uint32_t _i;
-        for( _i = 0; _i < _copy_count; _i++ ) {
-            OPAL_DATATYPE_SAFEGUARD_POINTER( _source, _copy_blength, (SOURCE_BASE),
-                                        (DATATYPE), (TOTAL_COUNT) );
-            DO_DEBUG( opal_output( 0, "copy 2. %s( %p, %p, %lu ) => space %lu\n",
-                                   STRINGIFY(MEM_OP_NAME), (void*)_destination, (void*)_source, (unsigned long)_copy_blength, (unsigned long)(*(SPACE) - (_i * _copy_blength)) ); );
-            MEM_OP( _destination, _source, _copy_blength );
-            _source      += _elem->extent;
-            _destination += _elem->extent;
-        }
-        _copy_blength *= _copy_count;
+    /* We don't a prologue and epilogue here as we are __always__ working
+     * with full copies of the data description.
+     */
+
+    /**
+     * Compute how many full blocklen we need to do and do them.
+     */
+    do_now_bytes = _elem->blocklen * opal_datatype_basicDatatypes[_elem->common.type]->size;
+    assert( (do_now * do_now_bytes) <= (*SPACE) );
+
+    for(size_t _i = 0; _i < do_now; _i++ ) {
+        OPAL_DATATYPE_SAFEGUARD_POINTER( _source, do_now_bytes, (SOURCE_BASE),
+                                         (DATATYPE), (TOTAL_COUNT) );
+        DO_DEBUG( opal_output( 0, "copy %s( %p, %p, %" PRIsize_t " ) => space %" PRIsize_t "\n",
+                               STRINGIFY(MEM_OP_NAME), (void*)_destination, (void*)_source, do_now_bytes, *(SPACE) - _i * do_now_bytes ); );
+        MEM_OP( _destination, _source, do_now_bytes );
+        _destination += _elem->extent;
+        _source      += _elem->extent;
     }
-    *(SPACE)      -= _copy_blength;
+    *(SPACE) -= (do_now_bytes * do_now);
 }
 
 static inline void _contiguous_loop( const dt_elem_desc_t* ELEM,
                                      const opal_datatype_t* DATATYPE,
                                      unsigned char* SOURCE_BASE,
                                      size_t TOTAL_COUNT,
-                                     uint32_t COUNT,
+                                     size_t COUNT,
                                      unsigned char* SOURCE,
                                      unsigned char* DESTINATION,
                                      size_t* SPACE )
@@ -96,7 +91,6 @@ static inline void _contiguous_loop( const dt_elem_desc_t* ELEM,
     unsigned char* _source = (SOURCE) + _end_loop->first_elem_disp;
     unsigned char* _destination = (DESTINATION) + _end_loop->first_elem_disp;
     size_t _copy_loops = (COUNT);
-    uint32_t _i;
 
     if( _loop->extent == (ptrdiff_t)_end_loop->size ) {  /* the loop is contiguous */
         _copy_loops *= _end_loop->size;
@@ -104,11 +98,11 @@ static inline void _contiguous_loop( const dt_elem_desc_t* ELEM,
                                     (DATATYPE), (TOTAL_COUNT) );
         MEM_OP( _destination, _source, _copy_loops );
     } else {
-        for( _i = 0; _i < _copy_loops; _i++ ) {
+        for(size_t _i = 0; _i < _copy_loops; _i++ ) {
             OPAL_DATATYPE_SAFEGUARD_POINTER( _source, _end_loop->size, (SOURCE_BASE),
                                         (DATATYPE), (TOTAL_COUNT) );
-            DO_DEBUG( opal_output( 0, "copy 3. %s( %p, %p, %lu ) => space %lu\n",
-                                   STRINGIFY(MEM_OP_NAME), (void*)_destination, (void*)_source, (unsigned long)_end_loop->size, (unsigned long)(*(SPACE) - _i * _end_loop->size) ); );
+            DO_DEBUG( opal_output( 0, "copy 3. %s( %p, %p, %" PRIsize_t " ) => space %" PRIsize_t "\n",
+                                   STRINGIFY(MEM_OP_NAME), (void*)_destination, (void*)_source, _end_loop->size, *(SPACE) - _i * _end_loop->size ); );
             MEM_OP( _destination, _source, _end_loop->size );
             _source      += _loop->extent;
             _destination += _loop->extent;
@@ -134,7 +128,7 @@ static inline int32_t _copy_content_same_ddt( const opal_datatype_t* datatype, i
     DO_DEBUG( opal_output( 0, "_copy_content_same_ddt( %p, %d, dst %p, src %p )\n",
                            (void*)datatype, count, (void*)destination_base, (void*)source_base ); );
 
-    iov_len_local = count * datatype->size;
+    iov_len_local = (size_t)count * datatype->size;
 
     /* If we have to copy a contiguous datatype then simply
      * do a MEM_OP.
@@ -149,12 +143,10 @@ static inline int32_t _copy_content_same_ddt( const opal_datatype_t* datatype, i
         if( (ptrdiff_t)datatype->size == extent ) {  /* all contiguous == no gaps around */
             size_t total_length = iov_len_local;
             size_t memop_chunk = opal_datatype_memop_block_size;
+            OPAL_DATATYPE_SAFEGUARD_POINTER( source, iov_len_local,
+                                             (unsigned char*)source_base, datatype, count );
             while( total_length > 0 ) {
                 if( memop_chunk > total_length ) memop_chunk = total_length;
-                OPAL_DATATYPE_SAFEGUARD_POINTER( destination, memop_chunk,
-                                            (unsigned char*)destination_base, datatype, count );
-                OPAL_DATATYPE_SAFEGUARD_POINTER( source, memop_chunk,
-                                            (unsigned char*)source_base, datatype, count );
                 DO_DEBUG( opal_output( 0, "copy c1. %s( %p, %p, %lu ) => space %lu\n",
                                        STRINGIFY(MEM_OP_NAME), (void*)destination, (void*)source, (unsigned long)memop_chunk, (unsigned long)total_length ); );
                 MEM_OP( destination, source, memop_chunk );
@@ -186,17 +178,12 @@ static inline int32_t _copy_content_same_ddt( const opal_datatype_t* datatype, i
     pos_desc = 0;
     stack_pos = 0;
 
-    if( datatype->opt_desc.desc != NULL ) {
-        description = datatype->opt_desc.desc;
-    } else {
+    description = datatype->opt_desc.desc;
+    if( NULL == description ) {
         description = datatype->desc.desc;
     }
 
-    if( description[0].elem.common.type == OPAL_DATATYPE_LOOP )
-        count_desc = description[0].loop.loops;
-    else
-        count_desc = description[0].elem.count;
-    pElem = &(description[pos_desc]);
+    UPDATE_INTERNAL_COUNTERS( description, 0, pElem, count_desc );
 
     while( 1 ) {
         while( OPAL_LIKELY(pElem->elem.common.flags & OPAL_DATATYPE_FLAG_DATA) ) {
@@ -207,8 +194,8 @@ static inline int32_t _copy_content_same_ddt( const opal_datatype_t* datatype, i
             UPDATE_INTERNAL_COUNTERS( description, pos_desc, pElem, count_desc );
         }
         if( OPAL_DATATYPE_END_LOOP == pElem->elem.common.type ) { /* end of the current loop */
-            DO_DEBUG( opal_output( 0, "copy end_loop count %d stack_pos %d pos_desc %d disp %ld space %lu\n",
-                                   (int)pStack->count, stack_pos, pos_desc, (long)pStack->disp, (unsigned long)iov_len_local ); );
+            DO_DEBUG( opal_output( 0, "copy end_loop count %" PRIsize_t " stack_pos %d pos_desc %d disp %ld space %lu\n",
+                                   pStack->count, stack_pos, pos_desc, pStack->disp, (unsigned long)iov_len_local ); );
             if( --(pStack->count) == 0 ) { /* end of loop */
                 if( stack_pos == 0 ) {
                     assert( iov_len_local == 0 );
@@ -229,8 +216,8 @@ static inline int32_t _copy_content_same_ddt( const opal_datatype_t* datatype, i
             source      = (unsigned char*)source_base + pStack->disp;
             destination = (unsigned char*)destination_base + pStack->disp;
             UPDATE_INTERNAL_COUNTERS( description, pos_desc, pElem, count_desc );
-            DO_DEBUG( opal_output( 0, "copy new_loop count %d stack_pos %d pos_desc %d disp %ld space %lu\n",
-                                   (int)pStack->count, stack_pos, pos_desc, (long)pStack->disp, (unsigned long)iov_len_local ); );
+            DO_DEBUG( opal_output( 0, "copy new_loop count %" PRIsize_t " stack_pos %d pos_desc %d disp %ld space %lu\n",
+                                   pStack->count, stack_pos, pos_desc, pStack->disp, (unsigned long)iov_len_local ); );
         }
         if( OPAL_DATATYPE_LOOP == pElem->elem.common.type ) {
             ptrdiff_t local_disp = (ptrdiff_t)source;
