@@ -19,6 +19,8 @@
 #include "btl_ugni.h"
 #include "btl_ugni_endpoint.h"
 
+#include <string.h>
+
 typedef struct mca_btl_ugni_send_frag_hdr_t {
     uint32_t lag;
 } mca_btl_ugni_send_frag_hdr_t;
@@ -83,59 +85,97 @@ typedef struct mca_btl_ugni_base_frag_t mca_btl_ugni_smsg_frag_t;
 typedef struct mca_btl_ugni_base_frag_t mca_btl_ugni_rdma_frag_t;
 typedef struct mca_btl_ugni_base_frag_t mca_btl_ugni_eager_frag_t;
 
-#define MCA_BTL_UGNI_DESC_TO_FRAG(desc) \
-    ((mca_btl_ugni_base_frag_t *)((uintptr_t) (desc) - offsetof (mca_btl_ugni_base_frag_t, post_desc)))
-
 typedef struct mca_btl_ugni_post_descriptor_t {
-    opal_free_list_item_t super;
-    gni_post_descriptor_t desc;
-    mca_btl_ugni_endpoint_handle_t *ep_handle;
+    /** endpoint currently associated with this desctiptor */
     mca_btl_base_endpoint_t *endpoint;
+    /** local memory handle (for callback) */
     mca_btl_base_registration_handle_t *local_handle;
-    mca_btl_base_rdma_completion_fn_t cbfunc;
+    /** currently associated completion queue */
     mca_btl_ugni_cq_t *cq;
+    /** user callback function */
+    mca_btl_base_rdma_completion_fn_t cbfunc;
+    /** user callback data */
     void *cbdata;
+    /** user callback context */
     void *ctx;
-    int tries;
+    /** opal status of this descriptor. filled in by
+     * mca_btl_ugni_cq_get_completed_desc_device() */
+    int rc;
+    /** true if posted with the BTE. false if FMA. this is used as part
+     * of the BTE throttling code. */
+    bool use_bte;
+    /** uGNI library post descriptor. this is last in this structure
+     * to try to keep it hot in the cache after copying this descriptor
+     * into the allocated descritor. (post follows almost immediately
+     * after allocate. */
+    gni_post_descriptor_t gni_desc;
 } mca_btl_ugni_post_descriptor_t;
 
 OBJ_CLASS_DECLARATION(mca_btl_ugni_post_descriptor_t);
 
-#define MCA_BTL_UGNI_DESC_TO_PDESC(desc)                                \
-    ((mca_btl_ugni_post_descriptor_t *)((uintptr_t) (desc) - offsetof (mca_btl_ugni_post_descriptor_t, desc)))
+typedef struct mca_btl_ugni_rdma_desc_t {
+    opal_free_list_item_t super;
+    mca_btl_ugni_post_descriptor_t btl_ugni_desc;
+    mca_btl_ugni_device_t *device;
+    gni_ep_handle_t gni_handle;
+    int tries;
+} mca_btl_ugni_rdma_desc_t;
 
-static inline mca_btl_ugni_post_descriptor_t *
-mca_btl_ugni_alloc_post_descriptor (mca_btl_base_endpoint_t *endpoint, mca_btl_base_registration_handle_t *local_handle,
-                                    mca_btl_base_rdma_completion_fn_t cbfunc, void *cbcontext, void *cbdata)
+OBJ_CLASS_DECLARATION(mca_btl_ugni_rdma_desc_t);
+
+#define MCA_BTL_UGNI_GNI_DESC_TO_RDMA_DESC(desc) \
+    ((mca_btl_ugni_rdma_desc_t *) ((uintptr_t)(desc) - offsetof (mca_btl_ugni_rdma_desc_t, btl_ugni_desc) - offsetof (mca_btl_ugni_post_descriptor_t, gni_desc)))
+
+/**
+ * Initialize a RDMA descriptor
+ *
+ * @param[in]    item    free list item (must be of class mca_btl_ugni_rdma_desc_t)
+ * @param[in]    ctx     pointer to ugni device context
+ *
+ * This function initializes a mca_btl_ugni_rdma_desc_t for use. It allocates
+ * resources from the ugni library. This must be called before a RDMA
+ * descriptor can be used. Usually this is passed as an argument to
+ * opal_free_list_init().
+ */
+int mca_btl_ugni_rdma_desc_init (opal_free_list_item_t *item, void *ctx);
+
+/**
+ * @brief get an endpoint handle from a device's free list
+ *
+ * @param[in] ep      btl endpoint
+ * @param[in] device  btl device to use
+ * @param[in] use_bte whether this descriptor will be used with the BTE
+ *
+ * This function MUST be called with the device lock held. This was done over using
+ * the atomic free list to avoid unnecessary atomics in the critical path.
+ */
+static inline mca_btl_ugni_rdma_desc_t *
+mca_btl_ugni_alloc_rdma_desc (mca_btl_ugni_device_t *device, mca_btl_ugni_post_descriptor_t *ugni_desc, const bool use_bte)
 {
-    /* mca_btl_ugni_module_t *ugni_module = mca_btl_ugni_ep_btl (endpoint); */
-    mca_btl_ugni_post_descriptor_t *desc;
+    mca_btl_ugni_rdma_desc_t *desc = (mca_btl_ugni_rdma_desc_t *) opal_free_list_get_st (&device->rdma_descs);
+    mca_btl_ugni_endpoint_t *ep = ugni_desc->endpoint;
+    gni_return_t grc;
 
-    desc = OBJ_NEW(mca_btl_ugni_post_descriptor_t);
-    /* (mca_btl_ugni_post_descriptor_t *) opal_free_list_get (&ugni_module->post_descriptors); */
-    if (OPAL_UNLIKELY(NULL != desc)) {
-        desc->cbfunc        = cbfunc;
-        desc->ctx           = cbcontext;
-        desc->cbdata        = cbdata;
-        desc->local_handle  = local_handle;
-        desc->endpoint      = endpoint;
+    if (OPAL_LIKELY(NULL != desc)) {
+        grc = GNI_EpBind (desc->gni_handle, ep->ep_rem_addr, ep->ep_rem_id | device->dev_index);
+        if (OPAL_UNLIKELY(GNI_RC_SUCCESS != grc)) {
+            opal_free_list_return_st (&device->rdma_descs, &desc->super);
+            return NULL;
+        }
+
+        desc->device = device;
+        desc->tries = 0;
+        desc->btl_ugni_desc = *ugni_desc;
+        desc->btl_ugni_desc.use_bte = use_bte;
     }
 
     return desc;
 }
 
-static inline void mca_btl_ugni_return_post_descriptor (mca_btl_ugni_post_descriptor_t *desc)
+static inline void mca_btl_ugni_return_rdma_desc (mca_btl_ugni_rdma_desc_t *desc)
 {
-    /* mca_btl_ugni_module_t *ugni_module = mca_btl_ugni_ep_btl (desc->endpoint); */
-
-    if (NULL != desc->ep_handle) {
-        mca_btl_ugni_ep_return_rdma (desc->ep_handle);
-        /* desc->ep_handle = NULL; */
-    }
-
-    /* desc->cq = NULL; */
-    /* opal_free_list_return (&ugni_module->post_descriptors, &desc->super); */
-    free (desc);
+    (void) GNI_EpUnbind (desc->gni_handle);
+    opal_free_list_return_st (&desc->device->rdma_descs, &desc->super);
 }
 
 static inline void mca_btl_ugni_post_desc_complete (mca_btl_ugni_module_t *module, mca_btl_ugni_post_descriptor_t *desc, int rc)
@@ -144,12 +184,9 @@ static inline void mca_btl_ugni_post_desc_complete (mca_btl_ugni_module_t *modul
 
     if (NULL != desc->cbfunc) {
         /* call the user's callback function */
-        desc->cbfunc (&module->super, desc->endpoint, (void *)(intptr_t) desc->desc.local_addr,
+        desc->cbfunc (&module->super, desc->endpoint, (void *)(intptr_t) desc->gni_desc.local_addr,
                       desc->local_handle, desc->ctx, desc->cbdata, rc);
     }
-
-    /* the descriptor is no longer needed */
-    mca_btl_ugni_return_post_descriptor (desc);
 }
 
 OBJ_CLASS_DECLARATION(mca_btl_ugni_smsg_frag_t);
@@ -194,7 +231,7 @@ static inline bool mca_btl_ugni_frag_del_ref (mca_btl_ugni_base_frag_t *frag, in
 
     opal_atomic_mb ();
 
-    ref_cnt = OPAL_THREAD_ADD32(&frag->ref_cnt, -1);
+    ref_cnt = OPAL_THREAD_ADD_FETCH32(&frag->ref_cnt, -1);
     if (ref_cnt) {
         assert (ref_cnt > 0);
         return false;
