@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
- * Copyright (c) 2014-2019 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2014-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2017-2019 Research Organization for Information Science
  *                         and Technology (RIST).  All rights reserved.
  * Copyright (c) 2017      IBM Corporation. All rights reserved.
@@ -11,15 +11,15 @@
  *
  * $HEADER$
  */
-#include <src/include/pmix_config.h>
+#include "src/include/pmix_config.h"
 
-#include <pmix.h>
-#include <pmix_common.h>
-#include <pmix_server.h>
-#include <pmix_rename.h>
+#include "include/pmix.h"
+#include "include/pmix_common.h"
+#include "include/pmix_server.h"
 
 #include "src/threads/threads.h"
 #include "src/util/error.h"
+#include "src/util/name_fns.h"
 #include "src/util/output.h"
 
 #include "src/mca/bfrops/bfrops.h"
@@ -71,7 +71,7 @@ PMIX_EXPORT pmix_status_t PMIx_Notify_event(pmix_status_t status,
     }
 
     /* if we aren't connected, don't attempt to send */
-    if (!pmix_globals.connected) {
+    if (!pmix_globals.connected && PMIX_RANGE_PROC_LOCAL != range) {
         PMIX_RELEASE_THREAD(&pmix_global_lock);
         return PMIX_ERR_UNREACH;
     }
@@ -93,6 +93,7 @@ PMIX_EXPORT pmix_status_t PMIx_Notify_event(pmix_status_t status,
 static void notify_event_cbfunc(struct pmix_peer_t *pr, pmix_ptl_hdr_t *hdr,
                                 pmix_buffer_t *buf, void *cbdata)
 {
+    (void)hdr;
     pmix_status_t rc, ret;
     int32_t cnt = 1;
     pmix_cb_t *cb = (pmix_cb_t*)cbdata;
@@ -110,7 +111,7 @@ static void notify_event_cbfunc(struct pmix_peer_t *pr, pmix_ptl_hdr_t *hdr,
     PMIX_RELEASE(cb);
 }
 
-static pmix_status_t notify_event_cache(pmix_notify_caddy_t *cd)
+pmix_status_t pmix_notify_event_cache(pmix_notify_caddy_t *cd)
 {
     pmix_status_t rc;
     int j;
@@ -219,8 +220,12 @@ static pmix_status_t notify_server_of_event(pmix_status_t status,
     /* setup for our own local callbacks */
     chain = PMIX_NEW(pmix_event_chain_t);
     chain->status = status;
-    pmix_strncpy(chain->source.nspace, pmix_globals.myid.nspace, PMIX_MAX_NSLEN);
-    chain->source.rank = pmix_globals.myid.rank;
+    chain->range = range;
+    if (NULL == source) {
+        PMIX_LOAD_PROCID(&chain->source, pmix_globals.myid.nspace, pmix_globals.myid.rank);
+    } else {
+        PMIX_LOAD_PROCID(&chain->source, source->nspace, source->rank);
+    }
     /* we always leave space for event hdlr name and a callback object */
     chain->nallocated = ninfo + 2;
     PMIX_INFO_CREATE(chain->info, chain->nallocated);
@@ -231,14 +236,8 @@ static pmix_status_t notify_server_of_event(pmix_status_t status,
      * ourselves should someone later register for it */
     cd = PMIX_NEW(pmix_notify_caddy_t);
     cd->status = status;
-    if (NULL == source) {
-        pmix_strncpy(cd->source.nspace, "UNDEF", PMIX_MAX_NSLEN);
-        cd->source.rank = PMIX_RANK_UNDEF;
-    } else {
-        pmix_strncpy(cd->source.nspace, source->nspace, PMIX_MAX_NSLEN);
-        cd->source.rank = source->rank;
-    }
-    cd->range = range;
+    PMIX_LOAD_PROCID(&cd->source, chain->source.nspace, chain->source.rank);
+    cd->range = chain->range;
     if (0 < chain->ninfo) {
         cd->ninfo = chain->ninfo;
         PMIX_INFO_CREATE(cd->info, cd->ninfo);
@@ -264,12 +263,13 @@ static pmix_status_t notify_server_of_event(pmix_status_t status,
         memcpy(cd->affected, chain->affected, cd->naffected * sizeof(pmix_proc_t));
     }
     /* cache it */
-    rc = notify_event_cache(cd);
+    rc = pmix_notify_event_cache(cd);
     if (PMIX_SUCCESS != rc) {
         PMIX_ERROR_LOG(rc);
         PMIX_RELEASE(cd);
         goto cleanup;
     }
+    chain->cached = true;
 
     if (PMIX_RANGE_PROC_LOCAL != range && NULL != msg) {
         /* create a callback object as we need to pass it to the
@@ -327,6 +327,10 @@ static void progress_local_event_hdlr(pmix_status_t status,
     pmix_list_item_t *item;
     pmix_event_hdlr_t *nxt;
 
+    pmix_output_verbose(2, pmix_client_globals.event_output,
+                        "%s progressing local event",
+                        PMIX_NAME_PRINT(&pmix_globals.myid));
+
     /* aggregate the results per RFC0018 - first search the
      * prior chained results to see if any keys have been NULL'd
      * as this indicates that info struct should be removed */
@@ -620,8 +624,8 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
     bool found;
 
     pmix_output_verbose(2, pmix_client_globals.event_output,
-                        "%s:%d invoke_local_event_hdlr for status %s",
-                        pmix_globals.myid.nspace, pmix_globals.myid.rank,
+                        "%s invoke_local_event_hdlr for status %s",
+                        PMIX_NAME_PRINT(&pmix_globals.myid),
                         PMIx_Error_string(chain->status));
 
     /* sanity check */
@@ -629,6 +633,9 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
         /* should never happen as space must always be
          * reserved for handler name and callback object*/
         rc = PMIX_ERR_BAD_PARAM;
+        pmix_output_verbose(8, pmix_client_globals.event_output,
+                            "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                            __FILE__, __LINE__);
         goto complete;
     }
 
@@ -636,15 +643,25 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
     if (NULL != chain->targets) {
         found = false;
         for (i=0; i < chain->ntargets; i++) {
+            pmix_output_verbose(8, pmix_client_globals.event_output,
+                                "%s CHECKING TARGET %s",
+                                PMIX_NAME_PRINT(&pmix_globals.myid),
+                                PMIX_NAME_PRINT(&chain->targets[i]));
             if (PMIX_CHECK_PROCID(&chain->targets[i], &pmix_globals.myid)) {
                 found = true;
                 break;
             }
         }
         if (!found) {
+            pmix_output_verbose(8, pmix_client_globals.event_output,
+                                "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                                __FILE__, __LINE__);
             goto complete;
         }
     }
+    pmix_output_verbose(8, pmix_client_globals.event_output,
+                        "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                        __FILE__, __LINE__);
 
     /* if we registered a "first" handler, and it fits the given range,
      * then invoke it first */
@@ -656,6 +673,9 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
                                        chain->affected, chain->naffected)) {
             /* invoke the handler */
             chain->evhdlr = pmix_globals.events.first;
+            pmix_output_verbose(8, pmix_client_globals.event_output,
+                                "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                                __FILE__, __LINE__);
             goto invk;
         } else if (NULL != pmix_globals.events.first->codes) {
             /* need to check if this code is included in the array */
@@ -671,6 +691,9 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
             if (found && pmix_notify_check_range(&pmix_globals.events.first->rng, &chain->source)) {
                 /* invoke the handler */
                 chain->evhdlr = pmix_globals.events.first;
+                pmix_output_verbose(8, pmix_client_globals.event_output,
+                                    "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                                    __FILE__, __LINE__);
                 goto invk;
             }
         } else {
@@ -678,11 +701,17 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
             if (pmix_notify_check_range(&pmix_globals.events.first->rng, &chain->source)) {
                 /* invoke the handler */
                 chain->evhdlr = pmix_globals.events.first;
+                pmix_output_verbose(8, pmix_client_globals.event_output,
+                                    "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                                    __FILE__, __LINE__);
                 goto invk;
             }
         }
         /* get here if there is no match, so fall thru */
     }
+    pmix_output_verbose(8, pmix_client_globals.event_output,
+                        "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                        __FILE__, __LINE__);
 
     /* cycle thru the single-event registrations first */
     PMIX_LIST_FOREACH(evhdlr, &pmix_globals.events.single_events, pmix_event_hdlr_t) {
@@ -692,6 +721,9 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
                                            chain->affected, chain->naffected)) {
                 /* invoke the handler */
                 chain->evhdlr = evhdlr;
+                pmix_output_verbose(8, pmix_client_globals.event_output,
+                                    "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                                    __FILE__, __LINE__);
                 goto invk;
             }
         }
@@ -707,11 +739,17 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
                                                chain->affected, chain->naffected)) {
                     /* invoke the handler */
                     chain->evhdlr = evhdlr;
+                    pmix_output_verbose(8, pmix_client_globals.event_output,
+                                        "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                                        __FILE__, __LINE__);
                     goto invk;
                 }
             }
         }
     }
+    pmix_output_verbose(8, pmix_client_globals.event_output,
+                        "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                        __FILE__, __LINE__);
 
     /* if they didn't want it to go to a default handler, then ignore them */
     if (!chain->nondefault) {
@@ -722,6 +760,9 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
                                            chain->affected, chain->naffected)) {
                 /* invoke the handler */
                 chain->evhdlr = evhdlr;
+                pmix_output_verbose(8, pmix_client_globals.event_output,
+                                    "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                                    __FILE__, __LINE__);
                 goto invk;
             }
         }
@@ -737,23 +778,37 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
         if (1 == pmix_globals.events.last->ncodes &&
             pmix_globals.events.last->codes[0] == chain->status) {
             chain->evhdlr = pmix_globals.events.last;
+            pmix_output_verbose(8, pmix_client_globals.event_output,
+                                "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                                __FILE__, __LINE__);
             goto invk;
         } else if (NULL != pmix_globals.events.last->codes) {
             /* need to check if this code is included in the array */
             for (i=0; i < pmix_globals.events.last->ncodes; i++) {
                 if (pmix_globals.events.last->codes[i] == chain->status) {
                     chain->evhdlr = pmix_globals.events.last;
+                    pmix_output_verbose(8, pmix_client_globals.event_output,
+                                        "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                                        __FILE__, __LINE__);
                     goto invk;
                 }
             }
         } else {
             /* gets run for all codes */
             chain->evhdlr = pmix_globals.events.last;
+            pmix_output_verbose(8, pmix_client_globals.event_output,
+                                "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                                __FILE__, __LINE__);
             goto invk;
         }
     }
+    pmix_output_verbose(8, pmix_client_globals.event_output,
+                        "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+
+                        __FILE__, __LINE__);
 
     /* if we got here, then nothing was found */
+    rc = PMIX_ERR_NOT_FOUND;
 
   complete:
     /* we still have to call their final callback */
@@ -766,6 +821,9 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
 
 
   invk:
+    pmix_output_verbose(8, pmix_client_globals.event_output,
+                        "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                        __FILE__, __LINE__);
     /* start with the chain holding only the given info */
     chain->ninfo = chain->nallocated - 2;
 
@@ -806,6 +864,8 @@ static void local_cbfunc(pmix_status_t status, void *cbdata)
 
 static void _notify_client_event(int sd, short args, void *cbdata)
 {
+    (void)sd;
+    (void)args;
     pmix_notify_caddy_t *cd = (pmix_notify_caddy_t*)cbdata;
     pmix_regevents_info_t *reginfoptr;
     pmix_peer_events_info_t *pr;
@@ -849,7 +909,7 @@ static void _notify_client_event(int sd, short args, void *cbdata)
          * the message until all local procs have received it, or it ages to
          * the point where it gets pushed out by more recent events */
         PMIX_RETAIN(cd);
-        rc = notify_event_cache(cd);
+        rc = pmix_notify_event_cache(cd);
         if (PMIX_SUCCESS != rc) {
             PMIX_ERROR_LOG(rc);
         }
@@ -859,8 +919,10 @@ static void _notify_client_event(int sd, short args, void *cbdata)
      * against our registrations */
     chain = PMIX_NEW(pmix_event_chain_t);
     chain->status = cd->status;
-    pmix_strncpy(chain->source.nspace, cd->source.nspace, PMIX_MAX_NSLEN);
-    chain->source.rank = cd->source.rank;
+    if (holdcd) {
+        chain->cached = true;
+    }
+    PMIX_LOAD_PROCID(&chain->source, cd->source.nspace, cd->source.rank);
     /* we always leave space for a callback object and
      * the evhandler name. */
     chain->nallocated = cd->ninfo + 2;
@@ -1095,15 +1157,6 @@ pmix_status_t pmix_server_notify_client_of_event(pmix_status_t status,
                         "pmix_server: notify client of event %s",
                         PMIx_Error_string(status));
 
-    if (NULL != info) {
-        for (n=0; n < ninfo; n++) {
-            if (PMIX_CHECK_KEY(&info[n], PMIX_EVENT_PROXY) &&
-                PMIX_CHECK_PROCID(info[n].value.data.proc, &pmix_globals.myid)) {
-                return PMIX_OPERATION_SUCCEEDED;
-            }
-        }
-    }
-
     cd = PMIX_NEW(pmix_notify_caddy_t);
     cd->status = status;
     if (NULL == source) {
@@ -1212,6 +1265,8 @@ bool pmix_notify_check_affected(pmix_proc_t *interested, size_t ninterested,
 
 void pmix_event_timeout_cb(int fd, short flags, void *arg)
 {
+    (void)fd;
+    (void)flags;
     pmix_event_chain_t *ch = (pmix_event_chain_t*)arg;
 
     /* need to acquire the object from its originating thread */
@@ -1374,6 +1429,7 @@ static void chcon(pmix_event_chain_t *p)
     p->source.rank = PMIX_RANK_UNDEF;
     p->nondefault = false;
     p->endchain = false;
+    p->cached = false;
     p->targets = NULL;
     p->ntargets = 0;
     p->range = PMIX_RANGE_UNDEF;
