@@ -53,7 +53,7 @@ static int rte_finalize(void);
 orte_ess_base_module_t orte_ess_lsf_module = {
     rte_init,
     rte_finalize,
-    orte_ess_base_app_abort,
+    NULL,
     NULL /* ft_event */
 };
 
@@ -100,15 +100,6 @@ static int rte_init(void)
 
     }
 
-    /* otherwise, I must be an application process - use
-     * the default procedure to finish my setup
-     */
-    if (ORTE_SUCCESS != (ret = orte_ess_base_app_setup(false))) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_ess_base_app_setup";
-        goto error;
-    }
-
     return ORTE_SUCCESS;
 
 error:
@@ -137,14 +128,6 @@ static int rte_finalize(void)
             ORTE_ERROR_LOG(ret);
         }
         return ret;
-    } else {
-        /* otherwise, I must be an application process
-         * use the default procedure to finish
-         */
-        if (ORTE_SUCCESS != (ret = orte_ess_base_app_finalize())) {
-            ORTE_ERROR_LOG(ret);
-            return ret;
-        }
     }
 
     return ORTE_SUCCESS;;
