@@ -2,8 +2,8 @@
  * Copyright (c) 2012-2013 Los Alamos National Security, LLC.
  *                         All rights reserved
  * Copyright (c) 2013      Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2014      Intel, Inc. All rights reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2014-2017 Intel, Inc. All rights reserved.
+ * Copyright (c) 2015-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
@@ -49,6 +49,7 @@
 #include "orte/util/name_fns.h"
 #include "orte/util/proc_info.h"
 #include "orte/util/session_dir.h"
+#include "orte/util/threads.h"
 #include "orte/runtime/orte_globals.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/mca/grpcomm/base/base.h"
@@ -61,14 +62,6 @@
 
 static int raw_init(void);
 static int raw_finalize(void);
-static int raw_put(orte_filem_base_request_t *req);
-static int raw_put_nb(orte_filem_base_request_t *req);
-static int raw_get(orte_filem_base_request_t *req);
-static int raw_get_nb(orte_filem_base_request_t *req);
-static int raw_rm(orte_filem_base_request_t *req);
-static int raw_rm_nb(orte_filem_base_request_t *req);
-static int raw_wait(orte_filem_base_request_t *req);
-static int raw_wait_all(opal_list_t *reqs);
 static int raw_preposition_files(orte_job_t *jdata,
                                  orte_filem_completion_cbfunc_t cbfunc,
                                  void *cbdata);
@@ -76,20 +69,20 @@ static int raw_link_local_files(orte_job_t *jdata,
                                 orte_app_context_t *app);
 
 orte_filem_base_module_t mca_filem_raw_module = {
-    raw_init,
-    raw_finalize,
+    .filem_init = raw_init,
+    .filem_finalize = raw_finalize,
     /* we don't use any of the following */
-    raw_put,
-    raw_put_nb,
-    raw_get,
-    raw_get_nb,
-    raw_rm,
-    raw_rm_nb,
-    raw_wait,
-    raw_wait_all,
+    .put = orte_filem_base_none_put,
+    .put_nb = orte_filem_base_none_put_nb,
+    .get = orte_filem_base_none_get,
+    .get_nb = orte_filem_base_none_get_nb,
+    .rm = orte_filem_base_none_rm,
+    .rm_nb = orte_filem_base_none_rm_nb,
+    .wait = orte_filem_base_none_wait,
+    .wait_all = orte_filem_base_none_wait_all,
     /* now the APIs we *do* use */
-    raw_preposition_files,
-    raw_link_local_files
+    .preposition_files = raw_preposition_files,
+    .link_local_files = raw_link_local_files
 };
 
 static opal_list_t outbound_files;
@@ -105,6 +98,17 @@ static void recv_ack(int status, orte_process_name_t* sender,
                      void* cbdata);
 static void write_handler(int fd, short event, void *cbdata);
 
+static char *filem_session_dir(void)
+{
+    char *session_dir = orte_process_info.jobfam_session_dir;
+    if( NULL == session_dir ){
+        /* if no job family session dir was provided -
+         * use the job session dir */
+        session_dir = orte_process_info.job_session_dir;
+    }
+    return session_dir;
+}
+
 static int raw_init(void)
 {
     OBJ_CONSTRUCT(&incoming_files, opal_list_t);
@@ -134,7 +138,6 @@ static int raw_finalize(void)
 {
     opal_list_item_t *item;
 
-    orte_rml.recv_cancel(ORTE_NAME_WILDCARD, ORTE_RML_TAG_FILEM_BASE);
     while (NULL != (item = opal_list_remove_first(&incoming_files))) {
         OBJ_RELEASE(item);
     }
@@ -149,52 +152,11 @@ static int raw_finalize(void)
             OBJ_RELEASE(item);
         }
         OBJ_DESTRUCT(&positioned_files);
-        orte_rml.recv_cancel(ORTE_NAME_WILDCARD, ORTE_RML_TAG_FILEM_BASE_RESP);
     }
 
     return ORTE_SUCCESS;
 }
 
-static int raw_put(orte_filem_base_request_t *req)
-{
-    return ORTE_SUCCESS;
-}
-
-static int raw_put_nb(orte_filem_base_request_t *req)
-{
-    return ORTE_SUCCESS;
-}
-
-static int raw_get(orte_filem_base_request_t *req)
-{
-    return ORTE_SUCCESS;
-}
-
-static int raw_get_nb(orte_filem_base_request_t *req)
-{
-    return ORTE_SUCCESS;
-}
-
-static int raw_rm(orte_filem_base_request_t *req)
-{
-    return ORTE_SUCCESS;
-}
-
-static int raw_rm_nb(orte_filem_base_request_t *req)
-{
-    return ORTE_SUCCESS;
-}
-
-static int raw_wait(orte_filem_base_request_t *req)
-{
-    return ORTE_SUCCESS;
-}
-
-static int raw_wait_all(opal_list_t *reqs)
-{
-    return ORTE_SUCCESS;
-}
-
 static void xfer_complete(int status, orte_filem_raw_xfer_t *xfer)
 {
     orte_filem_raw_outbound_t *outbound = xfer->outbound;
@@ -577,8 +539,9 @@ static int raw_preposition_files(orte_job_t *jdata,
         opal_list_append(&outbound->xfers, &xfer->super);
         opal_event_set(orte_event_base, &xfer->ev, fd, OPAL_EV_READ, send_chunk, xfer);
         opal_event_set_priority(&xfer->ev, ORTE_MSG_PRI);
-        opal_event_add(&xfer->ev, 0);
         xfer->pending = true;
+        ORTE_POST_OBJECT(xfer);
+        opal_event_add(&xfer->ev, 0);
         OBJ_RELEASE(item);
     }
     OBJ_DESTRUCT(&fsets);
@@ -657,25 +620,26 @@ static int create_link(char *my_dir, char *path,
 static int raw_link_local_files(orte_job_t *jdata,
                                 orte_app_context_t *app)
 {
-    char *my_dir, *path=NULL;
+    char *session_dir, *path=NULL;
     orte_proc_t *proc;
-    char *prefix;
     int i, j, rc;
     orte_filem_raw_incoming_t *inbnd;
     opal_list_item_t *item;
     char **files=NULL, *bname, *filestring;
 
-    /* check my session directory for files I have received and
+    /* check my jobfam session directory for files I have received and
      * symlink them to the proc-level session directory of each
      * local process in the job
+     *
+     * TODO: @rhc - please check that I've correctly interpret your
+     *  intention here
      */
-    my_dir = opal_dirname(orte_process_info.job_session_dir);
-
-    /* setup */
-    if (NULL != orte_process_info.tmpdir_base) {
-        prefix = strdup(orte_process_info.tmpdir_base);
-    } else {
-        prefix = NULL;
+    session_dir = filem_session_dir();
+    if( NULL == session_dir){
+        /* we were unable to find any suitable directory */
+        rc = ORTE_ERR_BAD_PARAM;
+        ORTE_ERROR_LOG(rc);
+        return rc;
     }
 
     /* get the list of files this app wants */
@@ -692,10 +656,6 @@ static int raw_link_local_files(orte_job_t *jdata,
 
     /* if there are no files to link, then ignore this */
     if (NULL == files) {
-        free(my_dir);
-        if (NULL != prefix) {
-            free(prefix);
-        }
         return ORTE_SUCCESS;
     }
 
@@ -736,10 +696,8 @@ static int raw_link_local_files(orte_job_t *jdata,
                              ORTE_NAME_PRINT(&proc->name)));
 
         /* get the session dir name in absolute form */
-        path = NULL;
-        rc = orte_session_dir_get_name(&path, &prefix, NULL,
-                                       orte_process_info.nodename,
-                                       NULL, &proc->name);
+        path = orte_process_info.proc_session_dir;
+
         /* create it, if it doesn't already exist */
         if (OPAL_SUCCESS != (rc = opal_os_dirpath_create(path, S_IRWXU))) {
             ORTE_ERROR_LOG(rc);
@@ -747,11 +705,6 @@ static int raw_link_local_files(orte_job_t *jdata,
              * create it - either way, we are done
              */
             free(files);
-            if (NULL != prefix) {
-                free(prefix);
-            }
-            free(path);
-            free(my_dir);
             return rc;
         }
 
@@ -775,13 +728,8 @@ static int raw_link_local_files(orte_job_t *jdata,
                                              inbnd->file));
                         /* cycle thru the link points and create symlinks to them */
                         for (j=0; NULL != inbnd->link_pts[j]; j++) {
-                            if (ORTE_SUCCESS != (rc = create_link(my_dir, path, inbnd->link_pts[j]))) {
+                            if (ORTE_SUCCESS != (rc = create_link(session_dir, path, inbnd->link_pts[j]))) {
                                 ORTE_ERROR_LOG(rc);
-                                free(my_dir);
-                                free(path);
-                                if (NULL != prefix) {
-                                    free(prefix);
-                                }
                                 free(files);
                                 return rc;
                             }
@@ -796,13 +744,8 @@ static int raw_link_local_files(orte_job_t *jdata,
                 }
             }
         }
-        free(path);
     }
     opal_argv_free(files);
-    if (NULL != prefix) {
-        free(prefix);
-    }
-    free(my_dir);
     return ORTE_SUCCESS;
 }
 
@@ -815,6 +758,8 @@ static void send_chunk(int fd, short argc, void *cbdata)
     opal_buffer_t chunk;
     orte_grpcomm_signature_t *sig;
 
+    ORTE_ACQUIRE_OBJECT(rev);
+
     /* flag that event has fired */
     rev->pending = false;
 
@@ -826,6 +771,7 @@ static void send_chunk(int fd, short argc, void *cbdata)
 
         /* non-blocking, retry */
         if (EAGAIN == errno || EINTR == errno) {
+            ORTE_POST_OBJECT(rev);
             opal_event_add(&rev->ev, 0);
             return;
         }
@@ -902,8 +848,9 @@ static void send_chunk(int fd, short argc, void *cbdata)
         return;
     } else {
         /* restart the read event */
-        opal_event_add(&rev->ev, 0);
         rev->pending = true;
+        ORTE_POST_OBJECT(rev);
+        opal_event_add(&rev->ev, 0);
     }
 }
 
@@ -923,7 +870,8 @@ static void send_complete(char *file, int status)
         OBJ_RELEASE(buf);
         return;
     }
-    if (0 > (rc = orte_rml.send_buffer_nb(ORTE_PROC_MY_HNP, buf,
+    if (0 > (rc = orte_rml.send_buffer_nb(orte_mgmt_conduit,
+                                          ORTE_PROC_MY_HNP, buf,
                                           ORTE_RML_TAG_FILEM_BASE_RESP,
                                           orte_rml_send_callback, NULL))) {
         ORTE_ERROR_LOG(rc);
@@ -999,7 +947,7 @@ static void recv_files(int status, orte_process_name_t* sender,
                        opal_buffer_t* buffer, orte_rml_tag_t tag,
                        void* cbdata)
 {
-    char *file, *jobfam_dir;
+    char *file, *session_dir;
     int32_t nchunk, n, nbytes;
     unsigned char data[ORTE_FILEM_RAW_CHUNK_MAX];
     int rc;
@@ -1086,9 +1034,9 @@ static void recv_files(int status, orte_process_name_t* sender,
         incoming->top = strdup(tmp);
         free(tmp);
         /* define the full path to where we will put it */
-        jobfam_dir = opal_dirname(orte_process_info.job_session_dir);
-        incoming->fullpath = opal_os_path(false, jobfam_dir, file, NULL);
-        free(jobfam_dir);
+        session_dir = filem_session_dir();
+
+        incoming->fullpath = opal_os_path(false, session_dir, file, NULL);
 
         OPAL_OUTPUT_VERBOSE((1, orte_filem_base_framework.framework_output,
                              "%s filem:raw: opening target file %s",
@@ -1126,7 +1074,8 @@ static void recv_files(int status, orte_process_name_t* sender,
             }
         }
         free(tmp);
-        opal_event_set(orte_event_base, &incoming->ev, incoming->fd, OPAL_EV_WRITE, write_handler, incoming);
+        opal_event_set(orte_event_base, &incoming->ev, incoming->fd,
+                       OPAL_EV_WRITE, write_handler, incoming);
         opal_event_set_priority(&incoming->ev, ORTE_MSG_PRI);
     }
     /* create an output object for this data */
@@ -1145,8 +1094,9 @@ static void recv_files(int status, orte_process_name_t* sender,
 
     if (!incoming->pending) {
         /* add the event */
-        opal_event_add(&incoming->ev, 0);
         incoming->pending = true;
+        ORTE_POST_OBJECT(incoming);
+        opal_event_add(&incoming->ev, 0);
     }
 
     /* cleanup */
@@ -1164,6 +1114,8 @@ static void write_handler(int fd, short event, void *cbdata)
     char homedir[MAXPATHLEN];
     int rc;
 
+    ORTE_ACQUIRE_OBJECT(sink);
+
     OPAL_OUTPUT_VERBOSE((1, orte_filem_base_framework.framework_output,
                          "%s write:handler writing data to %d",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
@@ -1236,8 +1188,9 @@ static void write_handler(int fd, short event, void *cbdata)
                 /* leave the write event running so it will call us again
                  * when the fd is ready.
                  */
-                opal_event_add(&sink->ev, 0);
                 sink->pending = true;
+                ORTE_POST_OBJECT(sink);
+                opal_event_add(&sink->ev, 0);
                 return;
             }
             /* otherwise, something bad happened so all we can do is abort
@@ -1260,8 +1213,9 @@ static void write_handler(int fd, short event, void *cbdata)
             /* leave the write event running so it will call us again
              * when the fd is ready
              */
-            opal_event_add(&sink->ev, 0);
             sink->pending = true;
+            ORTE_POST_OBJECT(sink);
+            opal_event_add(&sink->ev, 0);
             return;
         }
         OBJ_RELEASE(output);
