@@ -12,7 +12,7 @@
  * Copyright (c) 2006-2019 Cisco Systems, Inc.  All rights reserved
  * Copyright (c) 2007-2015 Los Alamos National Security, LLC.  All rights
  *                         reserved.
- * Copyright (c) 2014-2017 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2014-2020 Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -267,6 +267,9 @@ static void launch_daemons(int fd, short args, void *cbdata)
     /* start one orted on each node */
     opal_argv_append(&argc, &argv, "--ntasks-per-node=1");
 
+    /* ensure Slurm adds all CPUs to this task */
+    putenv("SLURM_WHOLE=1");
+
     if (!orte_enable_recovery) {
         /* kill the job if any orteds die */
         opal_argv_append(&argc, &argv, "--kill-on-bad-exit");
