@@ -1,8 +1,6 @@
 /*
  * Copyright (c) 2013      Mellanox Technologies, Inc.
  *                         All rights reserved.
- * Copyright (c) 2015      Los Alamos National Security, LLC.
- *                         All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -12,6 +10,8 @@
 
 #include "oshmem_config.h"
 
+#include "opal/runtime/opal_cr.h"
+
 #include "oshmem/constants.h"
 #include "oshmem/include/shmem.h"
 #include "oshmem/runtime/runtime.h"
@@ -22,14 +22,9 @@
 #include "oshmem/shmem/c/profile/defines.h"
 #endif
 
-extern int oshmem_shmem_globalexit_status;
-
 void shmem_finalize(void)
 {
-    if (oshmem_shmem_globalexit_status != 0)
-    {
-        return;
-    }
+    OPAL_CR_FINALIZE_LIBRARY();
     oshmem_shmem_finalize();
 }
 
