@@ -37,14 +37,12 @@ ompi_fortran_integer4_t shmem_int4_finc_f(FORTRAN_POINTER_T target, MPI_Fint *pe
 {
     ompi_fortran_integer4_t out_value = 0;
     ompi_fortran_integer4_t value = 1;
-    oshmem_op_t* op = oshmem_op_sum_fint4;
 
-    MCA_ATOMIC_CALL(fadd(FPTR_2_VOID_PTR(target),
+    MCA_ATOMIC_CALL(fadd(oshmem_ctx_default, FPTR_2_VOID_PTR(target),
         (void *)&out_value,
-        (const void*)&value,
+        value,
         sizeof(out_value),
-        OMPI_FINT_2_INT(*pe),
-        op));
+        OMPI_FINT_2_INT(*pe)));
 
     return out_value;
 }
