@@ -35,10 +35,9 @@ void  shmem_int4_set_f(FORTRAN_POINTER_T target, FORTRAN_POINTER_T value, MPI_Fi
 {
     ompi_fortran_integer4_t out_value = 0;
 
-    MCA_ATOMIC_CALL(cswap(FPTR_2_VOID_PTR(target),
+    MCA_ATOMIC_CALL(swap(oshmem_ctx_default, FPTR_2_VOID_PTR(target),
         (void *)&out_value,
-        NULL,
-        FPTR_2_VOID_PTR(value),
+        FPTR_2_INT(value, sizeof(out_value)),
         sizeof(out_value),
         OMPI_FINT_2_INT(*pe)));
 }
